@@ -28,6 +28,9 @@ class NavitAdapter : public IMapAdapter
 {
     Q_OBJECT
     Q_INTERFACES(IMapAdapter)
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+    Q_PLUGIN_METADATA(IID "be.merkaartor.NavitAdapter" FILE "NavitAdapter.json")
+#endif
 
 public:
     NavitAdapter();
