@@ -101,4 +101,6 @@ void YahooLegalMapAdapter::setImageManager(IImageManager* anImageManager)
     theImageManager = anImageManager;
 }
 
+#if !(QT_VERSION >= QT_VERSION_CHECK(5,0,0))
 Q_EXPORT_PLUGIN2(MYahooBackgroundPlugin, YahooLegalMapAdapterFactory)
+#endif
