@@ -63,7 +63,7 @@ SetTagCommand::SetTagCommand(Feature* aF)
 SetTagCommand::SetTagCommand(Feature* aF, int idx, const QString& k, const QString& v, Layer* aLayer)
 : TagCommand(aF, aLayer), theIdx(idx), theK(k), theV(v)
 {
-    description = MainWindow::tr("Set Tag '%1=%2' on %3").arg(k).arg(v).arg(aF->description());
+    description = QApplication::tr("Set Tag '%1=%2' on %3").arg(k).arg(v).arg(aF->description());
     oldLayer = theFeature->layer();
     redo();
 }
@@ -71,7 +71,7 @@ SetTagCommand::SetTagCommand(Feature* aF, int idx, const QString& k, const QStri
 SetTagCommand::SetTagCommand(Feature* aF, const QString& k, const QString& v, Layer* aLayer)
 : TagCommand(aF, aLayer), theIdx(-1), theK(k), theV(v)
 {
-    description = MainWindow::tr("Set Tag '%1=%2' on %3").arg(k).arg(v).arg(aF->description());
+    description = QApplication::tr("Set Tag '%1=%2' on %3").arg(k).arg(v).arg(aF->description());
     oldLayer = theFeature->layer();
     redo();
 }
@@ -135,60 +135,65 @@ bool SetTagCommand::buildDirtyList(DirtyList& theList)
     return theList.update(theFeature);
 }
 
-bool SetTagCommand::toXML(QDomElement& xParent) const
+bool SetTagCommand::toXML(QXmlStreamWriter& stream) const
 {
     bool OK = true;
 
-    QDomElement e = xParent.ownerDocument().createElement("SetTagCommand");
-    xParent.appendChild(e);
-
-    e.setAttribute("oldkey", oldK);
-    e.setAttribute("xml:id", id());
-    e.setAttribute("feature", theFeature->xmlId());
-    e.setAttribute("idx", QString::number(theIdx));
-    e.setAttribute("key", theK);
-    e.setAttribute("value", theV);
-    e.setAttribute("oldvalue", oldV);
+    stream.writeStartElement("SetTagCommand");
+    stream.writeAttribute("oldkey", oldK);
+    stream.writeAttribute("xml:id", id());
+    stream.writeAttribute("feature", theFeature->xmlId());
+    stream.writeAttribute("idx", QString::number(theIdx));
+    stream.writeAttribute("key", theK);
+    stream.writeAttribute("value", theV);
+    stream.writeAttribute("oldvalue", oldV);
     if (theLayer)
-        e.setAttribute("layer", theLayer->id());
+        stream.writeAttribute("layer", theLayer->id());
     if (oldLayer)
-        e.setAttribute("oldlayer", oldLayer->id());
+        stream.writeAttribute("oldlayer", oldLayer->id());
 
-    Command::toXML(e);
+    Command::toXML(stream);
+    stream.writeEndElement();
 
     return OK;
 }
 
-SetTagCommand * SetTagCommand::fromXML(Document * d, QDomElement e)
+SetTagCommand * SetTagCommand::fromXML(Document * d, QXmlStreamReader& stream)
 {
     Feature* F;
-    if (!(F = d->getFeature(IFeature::FId(IFeature::All, e.attribute("feature").toLongLong())))) {
-        qDebug() << "SetTagCommand::fromXML: Undefined feature: " << e.attribute("feature");
+    if (!(F = d->getFeature(IFeature::FId(IFeature::All, stream.attributes().value("feature").toString().toLongLong())))) {
+        qDebug() << "SetTagCommand::fromXML: Undefined feature: " << stream.attributes().value("feature").toString();
         return NULL;
     }
 
     SetTagCommand* a = new SetTagCommand(F);
-    a->setId(e.attribute("xml:id"));
-    if (e.hasAttribute("oldkey"))
-        a->oldK = e.attribute("oldkey");
+    a->setId(stream.attributes().value("xml:id").toString());
+    if (stream.attributes().hasAttribute("oldkey"))
+        a->oldK = stream.attributes().value("oldkey").toString();
     a->theFeature = F;
-    a->theIdx = e.attribute("idx").toInt();
-    a->theK = e.attribute("key");
-    a->theV = e.attribute("value");
-    if (e.hasAttribute("oldvalue"))
-        a->oldV = e.attribute("oldvalue");
-    if (e.hasAttribute("layer"))
-        a->theLayer = d->getLayer(e.attribute("layer"));
+    a->theIdx = stream.attributes().value("idx").toString().toInt();
+    a->theK = stream.attributes().value("key").toString();
+    a->theV = stream.attributes().value("value").toString();
+    if (stream.attributes().hasAttribute("oldvalue"))
+        a->oldV = stream.attributes().value("oldvalue").toString();
+    if (stream.attributes().hasAttribute("layer"))
+        a->theLayer = d->getLayer(stream.attributes().value("layer").toString());
     else
         a->theLayer = NULL;
-    if (e.hasAttribute("oldlayer"))
-        a->oldLayer = d->getLayer(e.attribute("oldlayer"));
+    if (stream.attributes().hasAttribute("oldlayer"))
+        a->oldLayer = d->getLayer(stream.attributes().value("oldlayer").toString());
     else
         a->oldLayer = NULL;
 
-    a->description = MainWindow::tr("Set Tag '%1=%2' on %3").arg(a->theK).arg(a->theV).arg(a->theFeature->description());
+    a->description = QApplication::tr("Set Tag '%1=%2' on %3").arg(a->theK).arg(a->theV).arg(a->theFeature->description());
 
-    Command::fromXML(d, e, a);
+    stream.readNext();
+    while(!stream.atEnd() && !stream.isEndElement()) {
+        if (stream.name() == "Command") {
+            Command::fromXML(d, stream, a);
+        }
+        stream.readNext();
+    }
 
     return a;
 }
@@ -235,61 +240,62 @@ void ClearTagsCommand::redo()
     Command::redo();
 }
 
-bool ClearTagsCommand::toXML(QDomElement& xParent) const
+bool ClearTagsCommand::toXML(QXmlStreamWriter& stream) const
 {
     bool OK = true;
 
-    QDomElement e = xParent.ownerDocument().createElement("ClearTagsCommand");
-    xParent.appendChild(e);
+    stream.writeStartElement("ClearTagsCommand");
 
-    e.setAttribute("xml:id", id());
-    e.setAttribute("feature", theFeature->xmlId());
+    stream.writeAttribute("xml:id", id());
+    stream.writeAttribute("feature", theFeature->xmlId());
     if (theLayer)
-        e.setAttribute("layer", theLayer->id());
+        stream.writeAttribute("layer", theLayer->id());
     if (oldLayer)
-        e.setAttribute("oldlayer", oldLayer->id());
+        stream.writeAttribute("oldlayer", oldLayer->id());
 
     for (int i=0; i<Before.size(); ++i)
     {
-        QDomElement c = e.ownerDocument().createElement("tag");
-        e.appendChild(c);
-
-        c.setAttribute("k", Before[i].first);
-        c.setAttribute("v", Before[i].second);
+        stream.writeStartElement("tag");
+        stream.writeAttribute("k", Before[i].first);
+        stream.writeAttribute("v", Before[i].second);
+        stream.writeEndElement();
     }
 
-    Command::toXML(e);
+    Command::toXML(stream);
+    stream.writeEndElement();
 
     return OK;
 }
 
-ClearTagsCommand * ClearTagsCommand::fromXML(Document * d, QDomElement e)
+ClearTagsCommand * ClearTagsCommand::fromXML(Document * d, QXmlStreamReader& stream)
 {
     Feature* F;
-    if (!(F = d->getFeature(IFeature::FId(IFeature::All, e.attribute("feature").toLongLong()))))
+    if (!(F = d->getFeature(IFeature::FId(IFeature::All, stream.attributes().value("feature").toString().toLongLong()))))
         return NULL;
 
     ClearTagsCommand* a = new ClearTagsCommand(F);
-    a->setId(e.attribute("xml:id"));
+    a->setId(stream.attributes().value("xml:id").toString());
     a->theFeature = F;
-    if (e.hasAttribute("layer"))
-        a->theLayer = d->getLayer(e.attribute("layer"));
+    if (stream.attributes().hasAttribute("layer"))
+        a->theLayer = d->getLayer(stream.attributes().value("layer").toString());
     else
         a->theLayer = NULL;
-    if (e.hasAttribute("oldlayer"))
-        a->oldLayer = d->getLayer(e.attribute("oldlayer"));
+    if (stream.attributes().hasAttribute("oldlayer"))
+        a->oldLayer = d->getLayer(stream.attributes().value("oldlayer").toString());
     else
         a->oldLayer = NULL;
 
-    QDomElement c = e.firstChildElement();
-    while(!c.isNull()) {
-        if (c.tagName() == "tag") {
-            a->Before.push_back(qMakePair(c.attribute("k"),c.attribute("v")));
+    stream.readNext();
+    while(!stream.atEnd() && !stream.isEndElement()) {
+        if (stream.name() == "tag") {
+            a->Before.push_back(qMakePair(stream.attributes().value("k").toString(),stream.attributes().value("v").toString()));
+        } else if (stream.name() == "Command") {
+            Command::fromXML(d, stream, a);
         }
-        c = c.nextSiblingElement();
+        stream.readNext();
     }
 
-    Command::fromXML(d, e, a);
+    Command::fromXML(d, stream, a);
 
     return a;
 }
@@ -305,7 +311,7 @@ ClearTagCommand::ClearTagCommand(Feature* F)
 ClearTagCommand::ClearTagCommand(Feature* F, const QString& k, Layer* aLayer)
 : TagCommand(F, aLayer), theIdx(F->findKey(k)), theK(k), theV(F->tagValue(k, ""))
 {
-    description = MainWindow::tr("Clear Tag '%1' on %2").arg(k).arg(F->description());
+    description = QApplication::tr("Clear Tag '%1' on %2").arg(k).arg(F->description());
     oldLayer = theFeature->layer();
     redo();
 }
@@ -348,52 +354,58 @@ bool ClearTagCommand::buildDirtyList(DirtyList& theList)
     return theList.update(theFeature);
 }
 
-bool ClearTagCommand::toXML(QDomElement& xParent) const
+bool ClearTagCommand::toXML(QXmlStreamWriter& stream) const
 {
     bool OK = true;
 
-    QDomElement e = xParent.ownerDocument().createElement("ClearTagCommand");
-    xParent.appendChild(e);
+    stream.writeStartElement("ClearTagCommand");
 
-    e.setAttribute("xml:id", id());
-    e.setAttribute("feature", theFeature->xmlId());
-    e.setAttribute("idx", QString::number(theIdx));
-    e.setAttribute("key", theK);
-    e.setAttribute("value", theV);
+    stream.writeAttribute("xml:id", id());
+    stream.writeAttribute("feature", theFeature->xmlId());
+    stream.writeAttribute("idx", QString::number(theIdx));
+    stream.writeAttribute("key", theK);
+    stream.writeAttribute("value", theV);
     if (theLayer)
-        e.setAttribute("layer", theLayer->id());
+        stream.writeAttribute("layer", theLayer->id());
     if (oldLayer)
-        e.setAttribute("oldlayer", oldLayer->id());
+        stream.writeAttribute("oldlayer", oldLayer->id());
 
-    Command::toXML(e);
+    Command::toXML(stream);
+    stream.writeEndElement();
 
     return OK;
 }
 
-ClearTagCommand * ClearTagCommand::fromXML(Document * d, QDomElement e)
+ClearTagCommand * ClearTagCommand::fromXML(Document * d, QXmlStreamReader& stream)
 {
     Feature* F;
-    if (!(F = d->getFeature(IFeature::FId(IFeature::All, e.attribute("feature").toLongLong()))))
+    if (!(F = d->getFeature(IFeature::FId(IFeature::All, stream.attributes().value("feature").toString().toLongLong()))))
         return NULL;
 
     ClearTagCommand* a = new ClearTagCommand(F);
-    a->setId(e.attribute("xml:id"));
+    a->setId(stream.attributes().value("xml:id").toString());
     a->theFeature = F;
-    a->theIdx = e.attribute("idx").toInt();
-    a->theK = e.attribute("key");
-    a->theV = e.attribute("value");
-    if (e.hasAttribute("layer"))
-        a->theLayer = d->getLayer(e.attribute("layer"));
+    a->theIdx = stream.attributes().value("idx").toString().toInt();
+    a->theK = stream.attributes().value("key").toString();
+    a->theV = stream.attributes().value("value").toString();
+    if (stream.attributes().hasAttribute("layer"))
+        a->theLayer = d->getLayer(stream.attributes().value("layer").toString());
     else
         a->theLayer = NULL;
-    if (e.hasAttribute("oldlayer"))
-        a->oldLayer = d->getLayer(e.attribute("oldlayer"));
+    if (stream.attributes().hasAttribute("oldlayer"))
+        a->oldLayer = d->getLayer(stream.attributes().value("oldlayer").toString());
     else
         a->oldLayer = NULL;
 
-    a->description = MainWindow::tr("Clear Tag '%1' on %2").arg(a->theK).arg(a->theFeature->description());
+    a->description = QApplication::tr("Clear Tag '%1' on %2").arg(a->theK).arg(a->theFeature->description());
 
-    Command::fromXML(d, e, a);
+    stream.readNext();
+    while(!stream.atEnd() && !stream.isEndElement()) {
+        if (stream.name() == "Command") {
+            Command::fromXML(d, stream, a);
+        }
+        stream.readNext();
+    }
 
     return a;
 }
