@@ -14,7 +14,7 @@ MoveNodeCommand::MoveNodeCommand(Node* aPt)
 {
     if (!theLayer)
         theLayer = thePoint->layer();
-    description = MainWindow::tr("Move node %1").arg(aPt->description());
+    description = QApplication::tr("Move node %1").arg(aPt->description());
 }
 
 MoveNodeCommand::MoveNodeCommand(Node* aPt, const Coord& aPos, Layer* aLayer)
@@ -22,7 +22,7 @@ MoveNodeCommand::MoveNodeCommand(Node* aPt, const Coord& aPos, Layer* aLayer)
 {
     if (!theLayer)
         theLayer = thePoint->layer();
-    description = MainWindow::tr("Move node %1").arg(aPt->description());
+    description = QApplication::tr("Move node %1").arg(aPt->description());
     redo();
 }
 
@@ -70,48 +70,57 @@ bool MoveNodeCommand::buildDirtyList(DirtyList &theList)
     return theList.noop(thePoint);
 }
 
-bool MoveNodeCommand::toXML(QDomElement& xParent) const
+bool MoveNodeCommand::toXML(QXmlStreamWriter& stream) const
 {
     bool OK = true;
 
-    QDomElement e = xParent.ownerDocument().createElement("MoveTrackPointCommand");
-    xParent.appendChild(e);
+    stream.writeStartElement("MoveTrackPointCommand");
 
-    e.setAttribute("xml:id", id());
-    e.setAttribute("trackpoint", thePoint->xmlId());
-    OldPos.toXML("oldpos", e);
-    NewPos.toXML("newpos", e);
+    stream.writeAttribute("xml:id", id());
+    stream.writeAttribute("trackpoint", thePoint->xmlId());
     if (theLayer)
-        e.setAttribute("layer", theLayer->id());
+        stream.writeAttribute("layer", theLayer->id());
     if (oldLayer)
-        e.setAttribute("oldlayer", oldLayer->id());
+        stream.writeAttribute("oldlayer", oldLayer->id());
+    OldPos.toXML("oldpos", stream);
+    NewPos.toXML("newpos", stream);
 
-    Command::toXML(e);
+    Command::toXML(stream);
+    stream.writeEndElement();
 
     return OK;
 }
 
-MoveNodeCommand * MoveNodeCommand::fromXML(Document * d, QDomElement e)
+MoveNodeCommand * MoveNodeCommand::fromXML(Document * d, QXmlStreamReader& stream)
 {
     MoveNodeCommand* a = new MoveNodeCommand();
-    a->setId(e.attribute("xml:id"));
-    a->OldPos = Coord::fromXML(e.firstChildElement("oldpos"));
-    a->NewPos = Coord::fromXML(e.firstChildElement("newpos"));
-    if (e.hasAttribute("layer"))
-        a->theLayer = d->getLayer(e.attribute("layer"));
+    a->setId(stream.attributes().value("xml:id").toString());
+
+    if (stream.attributes().hasAttribute("layer"))
+        a->theLayer = d->getLayer(stream.attributes().value("layer").toString());
     else
         a->theLayer = NULL;
-    if (e.hasAttribute("oldlayer"))
-        a->oldLayer = d->getLayer(e.attribute("oldlayer"));
+    if (stream.attributes().hasAttribute("oldlayer"))
+        a->oldLayer = d->getLayer(stream.attributes().value("oldlayer").toString());
     else
         a->oldLayer = NULL;
     if (!a->theLayer)
         return NULL;
 
-    a->thePoint = Feature::getTrackPointOrCreatePlaceHolder(d, a->theLayer, IFeature::FId(IFeature::Point, e.attribute("trackpoint").toLongLong()));
-    a->description = MainWindow::tr("Move node %1").arg(a->thePoint->description());
+    a->thePoint = Feature::getNodeOrCreatePlaceHolder(d, a->theLayer, IFeature::FId(IFeature::Point, stream.attributes().value("trackpoint").toString().toLongLong()));
+    a->description = QApplication::tr("Move node %1").arg(a->thePoint->description());
 
-    Command::fromXML(d, e, a);
+    stream.readNext();
+    while(!stream.atEnd() && !stream.isEndElement()) {
+        if (stream.name() == "oldpos") {
+            a->OldPos = Coord::fromXML(stream);
+        } else if (stream.name() == "newpos") {
+            a->NewPos = Coord::fromXML(stream);
+        } else if (stream.name() == "Command") {
+            Command::fromXML(d, stream, a);
+        }
+        stream.readNext();
+    }
 
     return a;
 }
