@@ -12,13 +12,13 @@
 #include <zbar/QZBarImage.h>
 #endif
 
-#include <QtGui/QInputDialog>
-#include <QtGui/QMessageBox>
-#include <QtGui/QProgressDialog>
+#include <QtWidgets/QInputDialog>
+#include <QtWidgets/QMessageBox>
+#include <QtWidgets/QProgressDialog>
 #include <QtGui/QClipboard>
-#include <QtGui/QRadioButton>
-#include <QtGui/QTimeEdit>
-#include <QtGui/QDialogButtonBox>
+#include <QtWidgets/QRadioButton>
+#include <QtWidgets/QTimeEdit>
+#include <QtWidgets/QDialogButtonBox>
 #include <QFileDialog>
 
 #include <QNetworkAccessManager>
@@ -27,6 +27,33 @@
 
 #include "ui_PhotoLoadErrorDialog.h"
 
+#define WARNING(title, message) { \
+    if (QMessageBox::warning(this, title, message.arg(file), \
+     QMessageBox::Ignore | QMessageBox::Cancel, QMessageBox::Ignore) == QMessageBox::Ignore) \
+        continue; \
+    else { \
+        theView->invalidate(true, true, false); \
+        return; \
+    } \
+}
+
+#define QUESTION(title, message, always) { \
+    if (always == 0) { \
+        int replyButton = QMessageBox::question(this, title, message, \
+         QMessageBox::Yes | QMessageBox::YesToAll | QMessageBox::No | QMessageBox::NoToAll | QMessageBox::Abort, QMessageBox::Yes ); \
+        if (replyButton == QMessageBox::No) \
+            continue; \
+        else if (replyButton == QMessageBox::Abort) { \
+            theView->invalidate(true, true, false); \
+            return; \
+        } \
+        else if (replyButton != QMessageBox::Yes) \
+            always = replyButton; \
+    } \
+    if (always == QMessageBox::NoToAll) \
+        continue; \
+}
+
 bool GeoImageDock::getWalkingPapersDetails(const QUrl& reqUrl, double &lat, double &lon, bool& positionValid) const
 {
     QNetworkAccessManager manager;
@@ -52,7 +79,7 @@ bool GeoImageDock::getWalkingPapersDetails(const QUrl& reqUrl, double &lat, doub
         return false;
     }
 
-    QString center = QString::fromAscii(reply->rawHeader("X-Print-Center"));
+    QString center = QString::fromLatin1(reply->rawHeader("X-Print-Center"));
     QStringList sl = center.split(" ");
     if (!sl.size() == 3)
         return false;
