@@ -73,28 +73,10 @@ IFeature::FId Feature::newId(IFeature::FeatureType type) const
 //		memcpy(&buffer[25+(i*2)], &hex[uuid.data4[i+2]], 2);
 //	}
 //
-//	return QString::fromAscii(buffer,38);
+//	return QString::fromLatin1(buffer,38);
 //#endif
 //}
 
-namespace boost
-{
-    void intrusive_ptr_add_ref(Feature * p)
-    {
-        ++(p->m_references);
-    }
-    void intrusive_ptr_release(Feature * p)
-    {
-        if (--(p->m_references) == 0) {
-            if (p->layer())
-                p->layer()->deleteFeature(p);
-            else
-                delete p;
-        }
-    }
-} // namespace boost
-
-
 void copyTags(Feature* Dest, Feature* Src)
 {
     for (int i=0; i<Src->tagSize(); ++i)
@@ -180,7 +162,7 @@ public:
 };
 
 Feature::Feature()
-: p(0), MetaUpToDate(false), m_references(0), ReadOnly(false)
+: MetaUpToDate(false), m_references(0), ReadOnly(false)
 {
     p = new FeaturePrivate(this);
     p->Id = IFeature::FId(IFeature::Uninitialized, 0);
@@ -618,6 +600,7 @@ void FeaturePrivate::updatePainters(qreal PixelPerM)
     if (!PossiblePaintersUpToDate)
         updatePossiblePainters();
 
+    QMutexLocker mutlock(&theFeature->featMutex);
     CurrentPainter = NULL;
     PixelPerMForPainter = PixelPerM;
     for (int i=0; i<PossiblePainters.size(); ++i)
