@@ -21,13 +21,6 @@
 #include <algorithm>
 #include <QList>
 
-#if (QT_VERSION < 0x040700 || defined(FORCE_46)) && !defined(_MOBILE)
-#include <ggl/ggl.hpp>
-#include <ggl/geometries/cartesian2d.hpp>
-#include <ggl/geometries/adapted/std_as_linestring.hpp>
-#include <ggl/algorithms/intersects.hpp>
-#endif
-
 #define TEST_RFLAGS(x) theView->renderOptions().options.testFlag(x)
 
 class WayPrivate
@@ -93,7 +86,7 @@ void WayPrivate::CalculateWidth()
         SimpleColor = QColor(228, 109, 113);
     } else if (h == "secondary") {
         SimpleWidth =  2*LANEWIDTH; // 2 lanes
-        SimpleColor = QColor(253, 191, 252);
+        SimpleColor = QColor(253, 191, 111);
     } else if (h == "tertiary") {
         SimpleWidth =  1.5*LANEWIDTH; // shared middle lane
         SimpleColor = QColor(255, 229, 33);
@@ -239,6 +232,7 @@ void Way::add(Node* Pt)
 
 void Way::add(Node* Pt, int Idx)
 {
+    QMutexLocker mutlock(&featMutex);
     p->Nodes.insert(p->Nodes.begin() + Idx, Pt);
 //	p->Nodes.push_back(Pt);
 //	std::rotate(p->Nodes.begin()+Idx,p->Nodes.end()-1,p->Nodes.end());
@@ -271,6 +265,7 @@ int Way::findVirtual(Feature* Pt) const
 
 void Way::remove(int idx)
 {
+    QMutexLocker mutlock(&featMutex);
     Node* Pt = p->Nodes[idx];
     // only remove as parent if the node is only included once
     p->Nodes.erase(p->Nodes.begin()+idx);
@@ -382,9 +377,8 @@ void Way::drawSimple(QPainter &P, MapView *theView)
                 thePen = QPen(thePainter->BackgroundColor,thePainter->BackgroundOffset);
             else
                 thePen = QPen(p->SimpleColor,(qreal)p->SimpleWidth/LANEWIDTH);
-        } else
-            if (thePainter->DrawForeground)
-                thePen = QPen(thePainter->ForegroundColor,thePainter->ForegroundOffset);
+        } else if (thePainter->DrawForeground)
+                thePen = QPen(thePainter->ForegroundColor,thePainter->ForegroundOffset >= 1 ? thePainter->ForegroundOffset : (qreal)p->SimpleWidth/LANEWIDTH);
             else
                 thePen = QPen(p->SimpleColor,(qreal)p->SimpleWidth/LANEWIDTH);
     }
@@ -406,6 +400,8 @@ void Way::updateMeta()
     p->Distance = 0;
     p->CalculateWidth();
 
+    MetaUpToDate = true;
+
     p->NotEverythingDownloaded = false;
     if (lastUpdated() == Feature::NotYetDownloaded)
         p->NotEverythingDownloaded = true;
@@ -448,7 +444,6 @@ void Way::updateMeta()
     }
 
     p->doUpdateVirtuals();
-    MetaUpToDate = true;
 }
 
 qreal Way::distance()
