@@ -589,6 +589,9 @@ bool QGPSDevice::parseGSV(const char *gsvString)
     currentSentence = tokens[2].toInt();
     totalSatellites = tokens[3].toInt();
 
+    qDebug() << "Parsing GSV string " << gsvString;
+    qDebug() << " --> sentence " << currentSentence << " of " << totalSentences << ", " << totalSatellites << " total satellites in view";
+
     for(int i = 0; (i < 4) && ((i*4)+4+3 < tokens.size()); i ++) {
         prn = tokens[(i*4)+4].toInt();
         elev = tokens[(i*4)+4+1].toInt();
@@ -958,27 +961,33 @@ void QGPSDDevice::run()
     connect(this,SIGNAL(doStopDevice()),&Forward,SLOT(onStop()));
 //    exec();
     QEventLoop l;
-
+#if GPSD_API_MAJOR_VERSION < 5
     Server = new gpsmm();
     errno = 0;
-    gpsdata = Server->open(M_PREFS->getGpsdHost().toAscii().data(),QString::number(M_PREFS->getGpsdPort()).toAscii().data());
+    gpsdata = Server->open(M_PREFS->getGpsdHost().toLatin1().data(),QString::number(M_PREFS->getGpsdPort()).toLatin1().data());
     if (!gpsdata) {
-#ifndef Q_OS_WIN32
+  #ifndef Q_OS_WIN32
         QString msg( (errno<0) ? gps_errstr(errno) : strerror(errno) );
-#else
+  #else
         QString msg( (errno<0) ? "" : strerror(errno) );
-#endif
+  #endif
         qDebug() << tr("Unable to connect to %1:%2").arg(M_PREFS->getGpsdHost()).arg(QString::number(M_PREFS->getGpsdPort()))
                  << ": " << msg;
         return;
     }
-
+#else
+    Server = new gpsmm(M_PREFS->getGpsdHost().toLatin1().data(),QString::number(M_PREFS->getGpsdPort()).toLatin1().data());
+#endif
     onLinkReady();
     l.processEvents();
 
     forever {
 #if GPSD_API_MAJOR_VERSION > 3
+#if GPSD_API_MAJOR_VERSION < 5
         if (Server->waiting())
+#else
+        if (Server->waiting(0))
+#endif
 #endif
             onDataAvailable();
         l.processEvents();
@@ -994,16 +1003,12 @@ void QGPSDDevice::onDataAvailable()
        if (!gpsdata)
            return;
     #else
-       if ( Server->waiting() )
+       gpsdata = Server->read();
+       if (!gpsdata)
            {
-           errno = 0;
-           gpsdata = Server->read();
-           if ( gpsdata == 0 )
-               {
-               QString msg( (errno==0) ? "socket to gpsd was closed" : strerror(errno) );
-               qDebug() << "gpsmm::read() failed: " << msg;
-               return;
-               }
+           QString msg( (errno==0) ? "socket to gpsd was closed" : strerror(errno) );
+           qDebug() << "gpsmm::read() failed: " << msg;
+           return;
            }
     #endif
 
