@@ -5,15 +5,16 @@
 #include "PropertiesDock.h"
 #include "DocumentCommands.h"
 #include "WayCommands.h"
-#include "Maps/Projection.h"
+#include "Projection.h"
 #include "Node.h"
-#include "Utils/LineF.h"
+#include "LineF.h"
 #include "MoveNodeInteraction.h"
+#include "Global.h"
 
 #include <QList>
 
-CreateNodeInteraction::CreateNodeInteraction(MapView* aView)
-    : FeatureSnapInteraction(aView)
+CreateNodeInteraction::CreateNodeInteraction(MainWindow* aMain)
+    : FeatureSnapInteraction(aMain)
     , theMoveInteraction(0)
 {
 }
@@ -47,7 +48,9 @@ void CreateNodeInteraction::snapMousePressEvent(QMouseEvent * ev, Feature* aFeat
         return theMoveInteraction->snapMousePressEvent(ev, aFeat);
     } else {
         SAFE_DELETE(theMoveInteraction);
-        theView->setCursor(cursor());
+#ifndef _MOBILE
+        theMain->view()->setCursor(cursor());
+#endif
     }
 
 }
@@ -56,11 +59,15 @@ void CreateNodeInteraction::snapMouseMoveEvent(QMouseEvent* ev, Feature* aFeat)
 {
     if (CAST_NODE(aFeat)) {
         if (!theMoveInteraction) {
-            theMoveInteraction = new MoveNodeInteraction(theView);
+            theMoveInteraction = new MoveNodeInteraction(theMain);
         }
-        theView->setCursor(theMoveInteraction->cursor());
+#ifndef _MOBILE
+        theMain->view()->setCursor(theMoveInteraction->cursor());
     } else
-        theView->setCursor(cursor());
+        theMain->view()->setCursor(cursor());
+#else
+    }
+#endif
 
     if (theMoveInteraction)
         return theMoveInteraction->snapMouseMoveEvent(ev, aFeat);
@@ -80,12 +87,14 @@ void CreateNodeInteraction::snapMouseReleaseEvent(QMouseEvent * ev, Feature* aFe
 
         createNode(P, aFeat);
 
-        theMoveInteraction = new MoveNodeInteraction(theView);
-        theView->setCursor(theMoveInteraction->cursor());
+        theMoveInteraction = new MoveNodeInteraction(theMain);
+#ifndef _MOBILE
+        theMain->view()->setCursor(theMoveInteraction->cursor());
+#endif
     }
 }
 
-#ifndef Q_OS_SYMBIAN
+#ifndef _MOBILE
 QCursor CreateNodeInteraction::cursor() const
 {
     return QCursor(Qt::CrossCursor);
@@ -102,13 +111,13 @@ void CreateNodeInteraction::createNode(Coord P, Feature* aFeat)
         g_Merk_MainWindow->properties()->setSelection(0);
         theList  = new CommandList(MainWindow::tr("Create node in Road: %1").arg(aRoad->id().numId), aRoad);
         int SnapIdx = findSnapPointIndex(aRoad, P);
-        N = new Node(P);
+        N = g_backend.allocNode(g_Merk_MainWindow->document()->getDirtyOrOriginLayer(aRoad->layer()), P);
         theList->add(new AddFeatureCommand(g_Merk_MainWindow->document()->getDirtyOrOriginLayer(aRoad->layer()),N,true));
         theList->add(new WayAddNodeCommand(aRoad,N,SnapIdx,g_Merk_MainWindow->document()->getDirtyOrOriginLayer(aRoad->layer())));
     }
     else
     {
-        N = new Node(P);
+        N = g_backend.allocNode(g_Merk_MainWindow->document()->getDirtyOrOriginLayer(), P);
         theList  = new CommandList(MainWindow::tr("Create POI %1").arg(N->id().numId), N);
         theList->add(new AddFeatureCommand(g_Merk_MainWindow->document()->getDirtyOrOriginLayer(),N,true));
         if (M_PREFS->getAutoSourceTag()) {
@@ -116,8 +125,9 @@ void CreateNodeInteraction::createNode(Coord P, Feature* aFeat)
             if (sl.size())
                 N->setTag("source", sl.join(";"));
         }
+        N->updateMeta();
     }
     g_Merk_MainWindow->document()->addHistory(theList);
     g_Merk_MainWindow->properties()->setSelection(N);
-    g_Merk_MainWindow->view()->invalidate(true, false);
+    g_Merk_MainWindow->view()->invalidate(true, true, false);
 }
