@@ -26,6 +26,7 @@ MoveNodeInteraction::MoveNodeInteraction(MainWindow* aMain)
     , StartDragPosition(0,0)
     , theList(0)
 {
+    setDontSelectVirtual(true);
     if (M_PREFS->getSeparateMoveMode()) {
         setDontSelectVirtual(false);
     }
@@ -271,6 +272,7 @@ void MoveNodeInteraction::snapMouseMoveEvent(QMouseEvent* event, Feature* Closer
                 theList->add(new WayAddNodeCommand(aRoad,N,SnapIdx,main()->document()->getDirtyOrOriginLayer(aRoad->layer())));
 
                 Moving[i] = N;
+                LastSnap = N;
             } else {
                 Moving[i]->setPosition(OriginalPosition[i]+Diff);
             }
