@@ -37,6 +37,7 @@ Layer::Layer()
 Layer::Layer(const QString& aName)
 :  p(new LayerPrivate), theWidget(0)
 {
+    qDebug() << "Created layer" << aName;
     p->Name = aName;
     p->alpha = 1.0;
     p->dirtyLevel = 0;
@@ -191,6 +192,13 @@ void Layer::clear()
     }
 }
 
+void Layer::deleteAll() {
+    while (p->Features.count())
+    {
+        deleteFeature(p->Features[0]);
+    }
+}
+
 void Layer::notifyIdUpdate(const IFeature::FId& id, Feature* aFeature)
 {
     QHash<qint64, MapFeaturePtr>::iterator i;
@@ -405,7 +413,7 @@ QString Layer::toMainHtml()
     "<small><i>" + QString(metaObject()->className()) + "</i></small><br/>"
     + desc;
     S += "<hr/>";
-    S += "<i>"+QApplication::translate("Layer", "Size")+": </i>" + QApplication::translate("Layer", "%n features", "", QCoreApplication::CodecForTr, getDisplaySize())+"<br/>";
+    S += "<i>"+QApplication::translate("Layer", "Size")+": </i>" + QApplication::translate("Layer", "%n features")+"<br/>";
     S += "%1";
     S += "</body></html>";
 
