@@ -1,10 +1,10 @@
-#include "MapView.h"
+#include "MapRenderer.h"
 #include "FeaturePainter.h"
-#include "Maps/Painting.h"
-#include "Maps/Projection.h"
+#include "Painting.h"
+#include "Projection.h"
 #include "Features.h"
-#include "Utils/LineF.h"
-#include "Utils/SvgCache.h"
+#include "LineF.h"
+#include "SvgCache.h"
 
 #include <QtCore/QString>
 #include <QtGui/QPainter>
@@ -13,7 +13,7 @@
 #include <QDomElement>
 #include <math.h>
 
-#define TEST_RFLAGS(x) theView->renderOptions().options.testFlag(x)
+#define TEST_RFLAGS(x) theRenderer->theOptions.options.testFlag(x)
 #define CAPSTYLE Qt::RoundCap
 #define JOINSTYLE Qt::RoundJoin
 
@@ -155,7 +155,7 @@ void FeaturePainter::setSelector(TagSelector* aSel)
     theSelector = aSel->asExpression(false);
 }
 
-TagSelectorMatchResult FeaturePainter::matchesTag(const Feature* F, const MapView* V) const
+TagSelectorMatchResult FeaturePainter::matchesTag(const Feature* F, MapRenderer* theRender) const
 {
     TagSelectorMatchResult res;
 
@@ -171,8 +171,8 @@ TagSelectorMatchResult FeaturePainter::matchesTag(const Feature* F, const MapVie
     //				return TagSelect_NoMatch;
     //	}
     //}
-    if (V)
-        res = theTagSelector->matches(F,V->pixelPerM());
+    if (theRender)
+        res = theTagSelector->matches(F,theRender->thePixelPerM);
     else
         res = theTagSelector->matches(F,0);
     if (res)
@@ -204,37 +204,77 @@ void buildPathFromRelation(Relation *R, const Projection &theProjection, QPainte
 
 */
 
-void FeaturePainter::drawBackground(Node* N, QPainter* thePainter, MapView* theView) const
+void FeaturePainter::drawBackground(Node* N, QPainter* thePainter, MapRenderer* theRenderer) const
 {
     if (!DrawBackground)
         return;
 
-    double PixelPerM = theView->pixelPerM();
-    double WW = PixelPerM*BackgroundScale+BackgroundOffset;
+    qreal PixelPerM = theRenderer->thePixelPerM;
+    qreal WW = PixelPerM*BackgroundScale+BackgroundOffset;
     if (WW >= 0)
     {
-        QPointF P(theView->toView(N));
+        QPointF P(theRenderer->toView(N));
         QRect R(P.x()-WW/2, P.y()-WW/2, WW, WW);
         thePainter->fillRect(R,BackgroundColor);
     }
 }
 
-void FeaturePainter::drawBackground(Way* R, QPainter* thePainter, MapView* theView) const
+void FeaturePainter::drawBackground(Way* R, QPainter* thePainter, MapRenderer* theRenderer) const
 {
     if (!DrawBackground && !ForegroundFill && !ForegroundFillUseIcon) return;
 
     thePainter->setPen(Qt::NoPen);
+    if (M_PREFS->getAreaOpacity() != 100 && ForegroundFill) {
+        thePainter->setOpacity(qreal(M_PREFS->getAreaOpacity()) / 100);
+    }
     if (DrawBackground)
     {
-        double PixelPerM = theView->pixelPerM();
-        double WW = PixelPerM*R->widthOf()*BackgroundScale+BackgroundOffset;
+        qreal WW = theRenderer->thePixelPerM*R->widthOf()*BackgroundScale+BackgroundOffset;
         if (WW >= 0)
         {
-            QPen thePen(BackgroundColor,WW);
-            thePen.setCapStyle(CAPSTYLE);
-            thePen.setJoinStyle(JOINSTYLE);
-            ////thePainter->strokePath(R->getPath(),thePen);
-            thePainter->setPen(thePen);
+            if (BackgroundExterior || BackgroundInterior) {
+                QPen thePen(BackgroundColor,WW);
+                thePen.setCapStyle(CAPSTYLE);
+                thePen.setJoinStyle(Qt::BevelJoin);
+                thePainter->setPen(thePen);
+
+                QPainterPath thePath = theRenderer->theTransform.map(R->getPath());
+                QPainterPath aPath;
+
+                for (int j=1; j < thePath.elementCount(); j++) {
+                    QLineF l(QPointF(thePath.elementAt(j)), QPointF(thePath.elementAt(j-1)));
+                    QLineF l1 = l.normalVector();
+                    if (BackgroundInterior)
+                        l1.setAngle(l1.angle() + 180.);
+                    l1.setLength(WW / 2.0);
+                    if (j == 1) {
+                        QLineF l3(l1);
+                        l3.translate(l.p2() - l.p1());
+                        aPath.moveTo(l3.p2());
+                    }
+                    if (j < thePath.elementCount() - 1) {
+                        QLineF l4(QPointF(thePath.elementAt(j)), QPointF(thePath.elementAt(j+1)));
+                        qreal theAngle = (l4.angle() - l.angle()) / 2.0;
+                        if (BackgroundInterior) {
+                            if (theAngle > 0.0) theAngle -= 180.0;
+                            l1.setLength(-1/sin(angToRad(theAngle))*l1.length());
+                        } else {
+                            if (theAngle < 0.0) theAngle += 180.0;
+                            l1.setLength(1/sin(angToRad(theAngle))*l1.length());
+                        }
+                        l1.setAngle(l.angle() + theAngle);
+                    }
+                    aPath.lineTo(l1.p2());
+                }
+                thePainter->drawPath(aPath);
+                thePainter->setPen(Qt::NoPen);
+            } else {
+                QPen thePen(BackgroundColor,WW);
+                thePen.setCapStyle(CAPSTYLE);
+                thePen.setJoinStyle(JOINSTYLE);
+                ////thePainter->strokePath(R->getPath(),thePen);
+                thePainter->setPen(thePen);
+            }
         }
     }
 
@@ -242,11 +282,11 @@ void FeaturePainter::drawBackground(Way* R, QPainter* thePainter, MapView* theVi
     if (R->size() > 2) {
         if (ForegroundFillUseIcon) {
             if (!IconName.isEmpty()) {
-                double PixelPerM = theView->pixelPerM();
-                double WW = PixelPerM*IconScale+IconOffset;
+                qreal PixelPerM = theRenderer->thePixelPerM;
+                qreal WW = PixelPerM*IconScale+IconOffset;
 
-                QPixmap* pm = getPixmapFromFile(IconName,int(WW));
-                if (!pm->isNull()) {
+                QImage* pm = getSVGImageFromFile(IconName,int(WW));
+                if (pm && !pm->isNull()) {
                     thePainter->setBrush(*pm);
                 }
             }
@@ -255,77 +295,110 @@ void FeaturePainter::drawBackground(Way* R, QPainter* thePainter, MapView* theVi
         }
     }
 
-    if (M_PREFS->getAreaOpacity() != 100 && ForegroundFill) {
-        thePainter->setOpacity(qreal(M_PREFS->getAreaOpacity()) / 100);
-    }
-    thePainter->drawPath(theView->transform().map(R->getPath()));
+    thePainter->drawPath(theRenderer->theTransform.map(R->getPath()));
 }
 
-void FeaturePainter::drawBackground(Relation* R, QPainter* thePainter, MapView* theView) const
+void FeaturePainter::drawBackground(Relation* R, QPainter* thePainter, MapRenderer* theRenderer) const
 {
     if (!DrawBackground && !ForegroundFill && !ForegroundFillUseIcon) return;
 
     thePainter->setPen(Qt::NoPen);
+    if (M_PREFS->getAreaOpacity() != 100 && ForegroundFill) {
+        thePainter->setOpacity(qreal(M_PREFS->getAreaOpacity()) / 100);
+    }
     if (DrawBackground)
     {
-        double PixelPerM = theView->pixelPerM();
-        double WW = PixelPerM*R->widthOf()*BackgroundScale+BackgroundOffset;
+        qreal PixelPerM = theRenderer->thePixelPerM;
+        qreal WW = PixelPerM*R->widthOf()*BackgroundScale+BackgroundOffset;
         if (WW >= 0)
         {
-            QPen thePen(BackgroundColor,WW);
-            thePen.setCapStyle(CAPSTYLE);
-            thePen.setJoinStyle(JOINSTYLE);
-            ////thePainter->strokePath(R->getPath(),thePen);
-            thePainter->setPen(thePen);
+            if (BackgroundExterior || BackgroundInterior) {
+                QPen thePen(BackgroundColor,WW);
+                thePen.setCapStyle(CAPSTYLE);
+                thePen.setJoinStyle(Qt::BevelJoin);
+                thePainter->setPen(thePen);
+
+                QPainterPath thePath = theRenderer->theTransform.map(R->getPath());
+                QPainterPath aPath;
+
+                for (int j=1; j < thePath.elementCount(); j++) {
+                    QLineF l(QPointF(thePath.elementAt(j)), QPointF(thePath.elementAt(j-1)));
+                    QLineF l1 = l.normalVector();
+                    if (BackgroundInterior)
+                        l1.setAngle(l1.angle() + 180.);
+                    l1.setLength(WW / 2.0);
+                    if (j == 1) {
+                        QLineF l3(l1);
+                        l3.translate(l.p2() - l.p1());
+                        aPath.moveTo(l3.p2());
+                    }
+                    if (j < thePath.elementCount() - 1) {
+                        QLineF l4(QPointF(thePath.elementAt(j)), QPointF(thePath.elementAt(j+1)));
+                        qreal theAngle = (l4.angle() - l.angle()) / 2.0;
+                        if (BackgroundInterior) {
+                            if (theAngle > 0.0) theAngle -= 180.0;
+                            l1.setLength(-1/sin(angToRad(theAngle))*l1.length());
+                        } else {
+                            if (theAngle < 0.0) theAngle += 180.0;
+                            l1.setLength(1/sin(angToRad(theAngle))*l1.length());
+                        }
+                        l1.setAngle(l.angle() + theAngle);
+                    }
+                    aPath.lineTo(l1.p2());
+                }
+                thePainter->drawPath(aPath);
+                thePainter->setPen(Qt::NoPen);
+            } else {
+                QPen thePen(BackgroundColor,WW);
+                thePen.setCapStyle(CAPSTYLE);
+                thePen.setJoinStyle(JOINSTYLE);
+                ////thePainter->strokePath(R->getPath(),thePen);
+                thePainter->setPen(thePen);
+            }
         }
     }
 
     thePainter->setBrush(Qt::NoBrush);
-    if (R->size() > 2) {
-        if (ForegroundFillUseIcon) {
-            if (!IconName.isEmpty()) {
-                double PixelPerM = theView->pixelPerM();
-                double WW = PixelPerM*IconScale+IconOffset;
+    if (ForegroundFillUseIcon) {
+        if (!IconName.isEmpty()) {
+            qreal PixelPerM = theRenderer->thePixelPerM;
+            qreal WW = PixelPerM*IconScale+IconOffset;
 
-                QPixmap* pm = getPixmapFromFile(IconName,int(WW));
-                if (!pm->isNull()) {
-                    thePainter->setBrush(*pm);
-                }
+            QImage* pm = getSVGImageFromFile(IconName,int(WW));
+            if (pm && !pm->isNull()) {
+                thePainter->setBrush(*pm);
             }
-        } else if (ForegroundFill) {
-            thePainter->setBrush(ForegroundFillFillColor);
         }
+    } else if (ForegroundFill) {
+        thePainter->setBrush(ForegroundFillFillColor);
     }
 
-    if (M_PREFS->getAreaOpacity() != 100 && ForegroundFill) {
-        thePainter->setOpacity(qreal(M_PREFS->getAreaOpacity()) / 100);
-    }
-    thePainter->drawPath(theView->transform().map(R->getPath()));
+    thePainter->drawPath(theRenderer->theTransform.map(R->getPath()));
 }
 
-void FeaturePainter::drawForeground(Node* N, QPainter* thePainter, MapView* theView) const
+void FeaturePainter::drawForeground(Node* N, QPainter* thePainter, MapRenderer* theRenderer) const
 {
     if (!DrawForeground)
         return;
 
-    double PixelPerM = theView->pixelPerM();
-    double WW = PixelPerM*ForegroundScale+ForegroundOffset;
+    qreal PixelPerM = theRenderer->thePixelPerM;
+    qreal WW = PixelPerM*ForegroundScale+ForegroundOffset;
     if (WW >= 0)
     {
-        QPointF P(theView->toView(N));
+        QPointF P(theRenderer->toView(N));
         QRect R(P.x()-WW/2, P.y()-WW/2, WW, WW);
         thePainter->fillRect(R,ForegroundColor);
     }
 }
 
-void FeaturePainter::drawForeground(Way* R, QPainter* thePainter, MapView* theView) const
+void FeaturePainter::drawForeground(Way* R, QPainter* thePainter, MapRenderer* theRenderer) const
 {
     if (!DrawForeground) return;
 
-    double WW = 0.0;
+    qreal WW = 0.0;
     if (DrawForeground)
     {
-        double PixelPerM = theView->pixelPerM();
+        qreal PixelPerM = theRenderer->thePixelPerM;
         WW = PixelPerM*R->widthOf()*ForegroundScale+ForegroundOffset;
         if (WW < 0) return;
         QPen thePen(ForegroundColor,WW);
@@ -344,17 +417,17 @@ void FeaturePainter::drawForeground(Way* R, QPainter* thePainter, MapView* theVi
 
     thePainter->setBrush(Qt::NoBrush);
 
-    thePainter->drawPath(theView->transform().map(R->getPath()));
+    thePainter->drawPath(theRenderer->theTransform.map(R->getPath()));
 }
 
-void FeaturePainter::drawForeground(Relation* R, QPainter* thePainter, MapView* theView) const
+void FeaturePainter::drawForeground(Relation* R, QPainter* thePainter, MapRenderer* theRenderer) const
 {
     if (!DrawForeground) return;
 
-    double WW = 0.0;
+    qreal WW = 0.0;
     if (DrawForeground)
     {
-        double PixelPerM = theView->pixelPerM();
+        qreal PixelPerM = theRenderer->thePixelPerM;
         WW = PixelPerM*R->widthOf()*ForegroundScale+ForegroundOffset;
         if (WW < 0) return;
         QPen thePen(ForegroundColor,WW);
@@ -373,26 +446,26 @@ void FeaturePainter::drawForeground(Relation* R, QPainter* thePainter, MapView*
 
     thePainter->setBrush(Qt::NoBrush);
 
-    thePainter->drawPath(theView->transform().map(R->getPath()));
+    thePainter->drawPath(theRenderer->theTransform.map(R->getPath()));
 }
 
 
-void FeaturePainter::drawTouchup(Node* Pt, QPainter* thePainter, MapView* theView) const
+void FeaturePainter::drawTouchup(Node* Pt, QPainter* thePainter, MapRenderer* theRenderer) const
 {
     bool IconOK = false;
     if (DrawIcon)
     {
         if (!IconName.isEmpty()) {
-            double PixelPerM = theView->pixelPerM();
-            double WW = PixelPerM*IconScale+IconOffset;
+            qreal PixelPerM = theRenderer->thePixelPerM;
+            qreal WW = PixelPerM*IconScale+IconOffset;
 
-            QPixmap* pm = getPixmapFromFile(IconName,int(WW));
-            if (!pm->isNull()) {
+            QImage* pm = getSVGImageFromFile(IconName,int(WW));
+            if (pm && !pm->isNull()) {
                 IconOK = true;
-                QPointF C(theView->transform().map(theView->projection().project(Pt)));
+                QPointF C(theRenderer->theTransform.map(Pt->projected()));
                 // cbro-20090109: Don't draw the dot if there is an icon
                 // thePainter->fillRect(QRect(C-QPoint(2,2),QSize(4,4)),QColor(0,0,0,128));
-                thePainter->drawPixmap( int(C.x()-pm->width()/2), int(C.y()-pm->height()/2) , *pm);
+                thePainter->drawImage( int(C.x()-pm->width()/2), int(C.y()-pm->height()/2) , *pm);
             }
         }
         if (!IconOK)
@@ -404,12 +477,15 @@ void FeaturePainter::drawTouchup(Node* Pt, QPainter* thePainter, MapView* theVie
                 if (DrawBackground)
                     theColor = BackgroundColor;
 
-            QPointF P(theView->toView(Pt));
-            double WW = theView->nodeWidth();
+            QPointF P(theRenderer->toView(Pt));
+            qreal WW = theRenderer->NodeWidth;
             if (WW >= 1) {
-                if (Pt->isWaypoint()) {
+                if (Pt->layer()->classGroups() & Layer::Special) {
                     QRect R2(P.x()-WW*4/3/2, P.y()-WW*4/3/2, WW*4/3, WW*4/3);
-                    thePainter->fillRect(R2,QColor(255,0,0,128));
+                    thePainter->fillRect(R2,QColor(255,0,255,192));
+                } else if (Pt->isWaypoint()) {
+                    QRect R2(P.x()-WW*4/3/2, P.y()-WW*4/3/2, WW*4/3, WW*4/3);
+                    thePainter->fillRect(R2,QColor(255,0,0,192));
                 }
 
                 QRect R(P.x()-WW/2, P.y()-WW/2, WW, WW);
@@ -419,12 +495,12 @@ void FeaturePainter::drawTouchup(Node* Pt, QPainter* thePainter, MapView* theVie
     }
 }
 
-void FeaturePainter::drawTouchup(Way* R, QPainter* thePainter, MapView* theView) const
+void FeaturePainter::drawTouchup(Way* R, QPainter* thePainter, MapRenderer* theRenderer) const
 {
     if (DrawTouchup)
     {
-        double PixelPerM = theView->pixelPerM();
-        double WW = PixelPerM*R->widthOf()*TouchupScale+TouchupOffset;
+        qreal PixelPerM = theRenderer->thePixelPerM;
+        qreal WW = PixelPerM*R->widthOf()*TouchupScale+TouchupOffset;
         if (WW > 0)
         {
             QPen thePen(TouchupColor,WW);
@@ -436,50 +512,50 @@ void FeaturePainter::drawTouchup(Way* R, QPainter* thePainter, MapView* theView)
                 Pattern << TouchupDash << TouchupWhite;
                 thePen.setDashPattern(Pattern);
             }
-            thePainter->strokePath(theView->transform().map(R->getPath()),thePen);
+            thePainter->strokePath(theRenderer->theTransform.map(R->getPath()),thePen);
         }
     }
     if (DrawIcon && !ForegroundFillUseIcon)
     {
         if (!IconName.isEmpty()) {
-            double PixelPerM = theView->pixelPerM();
-            double WW = PixelPerM*IconScale+IconOffset;
+            qreal PixelPerM = theRenderer->thePixelPerM;
+            qreal WW = PixelPerM*IconScale+IconOffset;
 
-            QPixmap* pm = getPixmapFromFile(IconName,int(WW));
-            if (!pm->isNull()) {
-                QPointF C(theView->transform().map(theView->projection().project(R->boundingBox().center())));
-                thePainter->drawPixmap( int(C.x()-pm->width()/2), int(C.y()-pm->height()/2) , *pm);
+            QImage* pm = getSVGImageFromFile(IconName,int(WW));
+            if (pm && !pm->isNull()) {
+                QPointF C(theRenderer->theTransform.map(R->getPath().boundingRect().center()));
+                thePainter->drawImage( int(C.x()-pm->width()/2), int(C.y()-pm->height()/2) , *pm);
             }
         }
     }
-    if ( ((DrawTrafficDirectionMarks) && (theView->renderOptions().arrowOptions == RendererOptions::ArrowsOneway)) ||  theView->renderOptions().arrowOptions == RendererOptions::ArrowsAlways)
+    if ( ((DrawTrafficDirectionMarks) && (theRenderer->theOptions.arrowOptions == RendererOptions::ArrowsOneway)) ||  theRenderer->theOptions.arrowOptions == RendererOptions::ArrowsAlways)
     {
         Feature::TrafficDirectionType TT = trafficDirection(R);
-        if ( (TT != Feature::UnknownDirection) || (theView->renderOptions().arrowOptions == RendererOptions::ArrowsAlways) )
+        if ( (TT != Feature::UnknownDirection) || (theRenderer->theOptions.arrowOptions == RendererOptions::ArrowsAlways) )
         {
-            double theWidth = theView->pixelPerM()*R->widthOf()-4;
+            qreal theWidth = theRenderer->thePixelPerM*R->widthOf()-4;
             if (theWidth > 8)
                 theWidth = 8;
-            double DistFromCenter = 2*(theWidth+4);
+            qreal DistFromCenter = 2*(theWidth+4);
             if (theWidth > 0)
             {
-                if ( theView->renderOptions().arrowOptions == RendererOptions::ArrowsAlways )
+                if ( theRenderer->theOptions.arrowOptions == RendererOptions::ArrowsAlways )
                     thePainter->setPen(QPen(QColor(255,0,0), 2));
                 else
-                    thePainter->setPen(QPen(QColor(0,0,255), 2));
+                    thePainter->setPen(QPen(TrafficDirectionMarksColor, 2));
 
 
                 for (int i=1; i<R->size(); ++i)
                 {
-                    QPointF FromF(theView->transform().map(theView->projection().project(R->getNode(i-1))));
-                    QPointF ToF(theView->transform().map(theView->projection().project(R->getNode(i))));
+                    QPointF FromF(theRenderer->theTransform.map(R->getNode(i-1)->projected()));
+                    QPointF ToF(theRenderer->theTransform.map(R->getNode(i)->projected()));
                     if (distance(FromF,ToF) > (DistFromCenter*2+4))
                     {
                         QPoint H(FromF.toPoint()+ToF.toPoint());
                         H *= 0.5;
-                        if (!theView->rect().contains(H))
+                        if (!theRenderer->theScreen.contains(H))
                             continue;
-                        double A = angle(FromF-ToF);
+                        qreal A = angle(FromF-ToF);
                         QPoint T(qRound(DistFromCenter*cos(A)),qRound(DistFromCenter*sin(A)));
                         QPoint V1(qRound(theWidth*cos(A+M_PI/6)),qRound(theWidth*sin(A+M_PI/6)));
                         QPoint V2(qRound(theWidth*cos(A-M_PI/6)),qRound(theWidth*sin(A-M_PI/6)));
@@ -495,7 +571,7 @@ void FeaturePainter::drawTouchup(Way* R, QPainter* thePainter, MapView* theView)
                         }
                         else
                         {
-                            if ( theView->renderOptions().arrowOptions == RendererOptions::ArrowsAlways )
+                            if ( theRenderer->theOptions.arrowOptions == RendererOptions::ArrowsAlways )
                             {
                                 thePainter->drawLine(H-T,H-T+V1);
                                 thePainter->drawLine(H-T,H-T+V2);
@@ -513,11 +589,11 @@ void FeaturePainter::drawTouchup(Way* R, QPainter* thePainter, MapView* theView)
 #define BG_SPACING 6
 #define BG_PEN_SZ 2
 
-void FeaturePainter::drawPointLabel(QPointF C, QString str, QString strBg, QPainter* thePainter, MapView* theView) const
+void FeaturePainter::drawPointLabel(QPointF C, QString str, QString strBg, QPainter* thePainter, MapRenderer* theRenderer) const
 {
     LineParameters lp = labelBoundary();
-    double PixelPerM = theView->pixelPerM();
-    double WW = PixelPerM*lp.Proportional+lp.Fixed;
+    qreal PixelPerM = theRenderer->thePixelPerM;
+    qreal WW = PixelPerM*lp.Proportional+lp.Fixed;
     if (WW < 10) return;
 
     QFont font = getLabelFont();
@@ -533,7 +609,7 @@ void FeaturePainter::drawPointLabel(QPointF C, QString str, QString strBg, QPain
         modX = - (metrics.width(str)/2);
         if (DrawIcon && (IconName != "") )
         {
-            QPixmap pm(IconName);
+            QImage pm(IconName);
             modY = - pm.height();
             if (DrawLabelBackground)
                 modY -= BG_SPACING;
@@ -545,7 +621,7 @@ void FeaturePainter::drawPointLabel(QPointF C, QString str, QString strBg, QPain
         modX = - (metrics.width(strBg)/2);
         if (DrawIcon && (IconName != "") )
         {
-            QPixmap pm(IconName);
+            QImage pm(IconName);
             modY = - pm.height();
             if (DrawLabelBackground)
                 modY -= BG_SPACING;
@@ -570,12 +646,12 @@ void FeaturePainter::drawPointLabel(QPointF C, QString str, QString strBg, QPain
     if (DrawLabelBackground && !strBg.isEmpty()) {
         QRegion rg = thePainter->clipRegion();
         rg -= textPath.boundingRect().toRect().translated(C.toPoint());
-        thePainter->setClipRegion(theView->transform().map(rg));
+        thePainter->setClipRegion(theRenderer->theTransform.map(rg));
     }
 }
 
 
-void FeaturePainter::drawLabel(Node* Pt, QPainter* thePainter, MapView* theView) const
+void FeaturePainter::drawLabel(Node* Pt, QPainter* thePainter, MapRenderer* theRenderer) const
 {
     if (!DrawLabel)
         return;
@@ -586,11 +662,11 @@ void FeaturePainter::drawLabel(Node* Pt, QPainter* thePainter, MapView* theView)
     if (str.isEmpty() && strBg.isEmpty())
         return;
 
-    QPointF C(theView->transform().map(theView->projection().project(Pt)));
-    drawPointLabel(C, str, strBg, thePainter, theView);
+    QPointF C(theRenderer->theTransform.map(Pt->projected()));
+    drawPointLabel(C, str, strBg, thePainter, theRenderer);
 }
 
-void FeaturePainter::drawLabel(Way* R, QPainter* thePainter, MapView* theView) const
+void FeaturePainter::drawLabel(Way* R, QPainter* thePainter, MapRenderer* theRenderer) const
 {
     if (!DrawLabel)
         return;
@@ -600,62 +676,62 @@ void FeaturePainter::drawLabel(Way* R, QPainter* thePainter, MapView* theView) c
     if (str.isEmpty() && strBg.isEmpty())
         return;
 
-    QRegion rg = thePainter->clipRegion();
     if (getLabelArea()) {
-        QPointF C(theView->transform().map(theView->projection().project(R->boundingBox().center())));
-        for (int i=0; i<rg.rects().size(); i++) {
-            if (rg.rects()[i].contains(C.toPoint())) {
-                drawPointLabel(C, str, strBg, thePainter, theView);
-            }
-        }
+        QPointF C(theRenderer->theTransform.map(R->getPath().boundingRect().center()));
+//        if (rg.contains(C.toPoint())) {
+            drawPointLabel(C, str, strBg, thePainter, theRenderer);
+//        }
         return;
     }
 
     LineParameters lp = labelBoundary();
-    double PixelPerM = theView->pixelPerM();
-    double WW = PixelPerM*R->widthOf()*lp.Proportional+lp.Fixed;
+    qreal PixelPerM = theRenderer->thePixelPerM;
+    qreal WW = PixelPerM*R->widthOf()*lp.Proportional+lp.Fixed;
     if (WW < 10 && !TEST_RFLAGS(RendererOptions::PrintAllLabels)) return;
-    //double WWR = qMax(PixelPerM*R->widthOf()*BackgroundScale+BackgroundOffset, PixelPerM*R->widthOf()*ForegroundScale+ForegroundOffset);
+    //qreal WWR = qMax(PixelPerM*R->widthOf()*BackgroundScale+BackgroundOffset, PixelPerM*R->widthOf()*ForegroundScale+ForegroundOffset);
 
-    QPainterPath tranformedRoadPath = theView->transform().map(R->getPath());
+    QPainterPath tranformedRoadPath = theRenderer->theTransform.map(R->getPath());
     QFont font = getLabelFont();
-#if QT_VERSION >= 0x040700
-    qreal pathSurface = tranformedRoadPath.controlPointRect().width() * tranformedRoadPath.controlPointRect().height();
-    if (pathSurface > theView->rect().width() * theView->rect().height() * 3) {
-        QPainterPath clipPath;
-        clipPath.addRect(theView->rect().adjusted(-500, -500, 500, 500));
+//#if QT_VERSION >= 0x040700 || defined(FORCE_46)
+//    qreal pathSurface = tranformedRoadPath.controlPointRect().width() * tranformedRoadPath.controlPointRect().height();
+//    if (pathSurface > theRenderer->theScreen.width() * theRenderer->theScreen.height() * 3) {
+//        QPainterPath clipPath;
+//        clipPath.addRect(theRenderer->theScreen.adjusted(-500, -500, 500, 500));
 
-        tranformedRoadPath = clipPath.intersected(tranformedRoadPath);
-    }
-#endif
+//        tranformedRoadPath = clipPath.intersected(tranformedRoadPath);
+//    }
+//#endif
 
     if (!str.isEmpty()) {
         font.setPixelSize(int(WW));
         QFontMetricsF metrics(font);
+        qreal strWidth = metrics.width(str);
 
-        if ((font.pixelSize() >= 5 || TEST_RFLAGS(RendererOptions::PrintAllLabels)) && tranformedRoadPath.length() > metrics.width(str)) {
+        if ((font.pixelSize() >= 5 || TEST_RFLAGS(RendererOptions::PrintAllLabels)) && tranformedRoadPath.length() > strWidth) {
             thePainter->setFont(font);
 
-            int repeat = int((tranformedRoadPath.length() / ((metrics.width(str) * LABEL_PATH_DISTANCE))) - 0.5);
+            int repeat = int((tranformedRoadPath.length() / ((strWidth * LABEL_PATH_DISTANCE))) - 0.5);
             int numSegment = repeat+1;
             qreal lenSegment = tranformedRoadPath.length() / numSegment;
             qreal startSegment = 0;
             QPainterPath textPath;
             do {
-                qreal curLen = startSegment + ((lenSegment - metrics.width(str)) / 2);
+                qreal curLen = startSegment + ((lenSegment - strWidth) / 2);
                 int modIncrement = 1;
                 qreal modAngle = 0;
                 qreal modY = 0;
                 if (cos(angToRad(tranformedRoadPath.angleAtPercent((startSegment+(lenSegment/2))/tranformedRoadPath.length()))) < 0) {
                     modIncrement = -1;
                     modAngle = 180.0;
-                    curLen += metrics.width(str);
+                    curLen += strWidth;
                 }
                 for (int i = 0; i < str.length(); ++i) {
                     qreal t = tranformedRoadPath.percentAtLength(curLen);
                     QPointF pt = tranformedRoadPath.pointAtPercent(t);
-                    if (!theView->rect().contains(pt.toPoint()))
-                        continue;
+
+//                    if (!theRenderer->theScreen.contains(pt.toPoint()))
+//                        continue;
+
                     qreal angle = tranformedRoadPath.angleAtPercent(t);
 //                    modY = (metrics.ascent()/2)-3;
                     modY = (metrics.height()/2)-metrics.descent();
@@ -683,20 +759,19 @@ void FeaturePainter::drawLabel(Way* R, QPainter* thePainter, MapView* theView) c
             thePainter->setPen(Qt::NoPen);
             thePainter->setBrush(LabelColor);
             thePainter->drawPath(textPath);
-            thePainter->setClipRegion(rg);
         }
     }
     if (DrawLabelBackground && !strBg.isEmpty()) {
         QRegion rg = thePainter->clipRegion();
         font.setPixelSize(int(WW));
         QFontMetrics metrics(font);
+        qreal strWidth = metrics.width(strBg);
 
-        int repeat = int((tranformedRoadPath.length() / (metrics.width(strBg) * LABEL_STRAIGHT_DISTANCE)) - 0.5);
+        int repeat = int((tranformedRoadPath.length() / (strWidth * LABEL_STRAIGHT_DISTANCE)) - 0.5);
         int numSegment = repeat+1;
         qreal lenSegment = tranformedRoadPath.length() / numSegment;
         qreal startSegment = 0;
         do {
-
             int modX = 0;
             int modY = 0;
 
@@ -704,7 +779,7 @@ void FeaturePainter::drawLabel(Way* R, QPainter* thePainter, MapView* theView) c
             qreal t = tranformedRoadPath.percentAtLength(curLen);
             QPointF pt = tranformedRoadPath.pointAtPercent(t);
 
-            modX = - (metrics.width(strBg)/2);
+            modX = - (strWidth/2);
             //modX = WW;
             modY = (metrics.ascent()/2);
 
@@ -712,14 +787,7 @@ void FeaturePainter::drawLabel(Way* R, QPainter* thePainter, MapView* theView) c
             textPath.addText(modX, modY, font, strBg);
             bgPath.addRect(textPath.boundingRect().adjusted(-BG_SPACING, -BG_SPACING, BG_SPACING, BG_SPACING));
 
-            bool rgContains = false;
-            for (int i=0; i<rg.rects().size(); i++) {
-                if (rg.rects()[i].contains(bgPath.boundingRect().toRect().translated(pt.toPoint()))) {
-                    rgContains = true;
-                    break;
-                }
-            }
-            if (rgContains) {
+            if (rg.contains(bgPath.boundingRect().toRect().translated(pt.toPoint()))) {
                 thePainter->translate(pt);
 
                 thePainter->setPen(QPen(LabelColor, BG_PEN_SZ));
