@@ -1,5 +1,5 @@
 #include "Painter.h"
-#include "Utils/SvgCache.h"
+#include "SvgCache.h"
 
 #include <QtCore/QString>
 #include <QtGui/QPainter>
@@ -16,10 +16,12 @@ Painter::Painter()
   ForegroundDashSet(false),
   DrawTouchup(false), TouchupScale(0), TouchupOffset(1),
   TouchupDashSet(false),
-  ForegroundFill(false), ForegroundFillUseIcon(false), DrawTrafficDirectionMarks(false),
+  ForegroundFill(false), ForegroundFillUseIcon(false),
+  DrawTrafficDirectionMarks(false),
   DrawIcon(false), IconScale(0), IconOffset(0),
   DrawLabel(false), LabelScale(0), LabelOffset(0),
-  DrawLabelBackground(false), LabelHalo(false), LabelArea(false)
+  DrawLabelBackground(false), LabelHalo(false), LabelArea(false),
+  BackgroundInterior(false), BackgroundExterior(false)
 {
 }
 
@@ -35,11 +37,12 @@ Painter::Painter(const Painter& f)
   TouchupDashSet(f.TouchupDashSet),
   TouchupDash(f.TouchupDash), TouchupWhite(f.TouchupWhite),
   ForegroundFill(f.ForegroundFill), ForegroundFillFillColor(f.ForegroundFillFillColor), ForegroundFillUseIcon(f.ForegroundFillUseIcon),
-  DrawTrafficDirectionMarks(f.DrawTrafficDirectionMarks),
+  DrawTrafficDirectionMarks(f.DrawTrafficDirectionMarks), TrafficDirectionMarksColor(f.TrafficDirectionMarksColor),
   DrawIcon(f.DrawIcon), IconName(f.IconName), IconScale(f.IconScale), IconOffset(f.IconOffset),
   DrawLabel(f.DrawLabel), LabelTag(f.LabelTag), LabelColor(f.LabelColor), LabelScale(f.LabelScale), LabelOffset(f.LabelOffset),
   DrawLabelBackground(f.DrawLabelBackground), LabelBackgroundColor(f.LabelBackgroundColor), LabelBackgroundTag(f.LabelBackgroundTag),
   LabelFont(f.LabelFont), LabelHalo(f.LabelHalo), LabelArea(f.LabelArea),
+  BackgroundInterior(f.BackgroundInterior), BackgroundExterior(f.BackgroundExterior),
   theSelector(f.theSelector)
 {
 }
@@ -73,6 +76,7 @@ Painter& Painter::operator=(const Painter& f)
     ForegroundFillFillColor = f.ForegroundFillFillColor;
     ForegroundFillUseIcon = f.ForegroundFillUseIcon;
     DrawTrafficDirectionMarks = f.DrawTrafficDirectionMarks;
+    TrafficDirectionMarksColor = f.TrafficDirectionMarksColor;
     DrawIcon = f.DrawIcon;
     IconName = f.IconName;
     IconScale = f.IconScale;
@@ -89,6 +93,8 @@ Painter& Painter::operator=(const Painter& f)
     LabelHalo = f.LabelHalo;
     LabelArea = f.LabelArea;
     theSelector = f.theSelector;
+    BackgroundInterior = f.BackgroundInterior;
+    BackgroundExterior = f.BackgroundExterior;
 
     return *this;
 }
@@ -116,13 +122,13 @@ QString colorAsXML(const QString& name, const QColor& c)
         name+"Color=\""+asXML(c)+"\"\n";
 }
 
-QString boundaryAsXML(const QString& name, const QColor& c, double Scale, double Offset)
+QString boundaryAsXML(const QString& name, const QColor& c, qreal Scale, qreal Offset)
 {
     return
         name+"Color=\""+asXML(c)+"\" "+name+"Scale=\""+QString::number(Scale)+"\" "+name+"Offset=\""+QString::number(Offset)+"\"\n";
 }
 
-QString iconAsXML(const QString& name, const QString& fn, double Scale, double Offset)
+QString iconAsXML(const QString& name, const QString& fn, qreal Scale, qreal Offset)
 {
     return
         name+"=\""+fn+"\" "+name+"Scale=\""+QString::number(Scale)+"\" "+name+"Offset=\""+QString::number(Offset)+"\"\n";
@@ -147,6 +153,10 @@ QString Painter::toXML(QString filename) const
         r += " zoomUnder=\""+QString::number(ZoomUnder)+"\" zoomUpper=\""+QString::number(ZoomUpper)+"\"\n";
     if (DrawBackground)
         r += " " + boundaryAsXML("background",BackgroundColor, BackgroundScale, BackgroundOffset);
+    if (BackgroundInterior)
+        r += " interior=\"yes\"";
+    if (BackgroundExterior)
+        r += " exterior=\"yes\"";
     if (DrawForeground)
         r += " " + boundaryAsXML("foreground",ForegroundColor, ForegroundScale, ForegroundOffset);
     if (ForegroundDashSet && DrawForeground)
@@ -170,9 +180,7 @@ QString Painter::toXML(QString filename) const
         r += " " + iconAsXML("icon",iconFilename, IconScale, IconOffset);
     }
     if (DrawTrafficDirectionMarks)
-        r += " drawTrafficDirectionMarks=\"yes\"";
-    else
-        r += " drawTrafficDirectionMarks=\"no\"";
+        r += " drawTrafficDirectionMarks=\"yes\" trafficDirectionMarksColor=\"" + ::asXML(TrafficDirectionMarksColor) +"\"\n";
     if (DrawLabel) {
         r += " " + boundaryAsXML("label",LabelColor, LabelScale, LabelOffset);
         r += " labelFont=\"" + LabelFont.toString() + "\"";
@@ -213,6 +221,10 @@ Painter Painter::fromXML(const QDomElement& e, QString filename)
     if (e.hasAttribute("backgroundColor"))
         FP.background(
             toColor(e.attribute("backgroundColor")),e.attribute("backgroundScale").toDouble(),e.attribute("backgroundOffset").toDouble());
+    if (e.attribute("interior") == "yes")
+        FP.BackgroundInterior = true;
+    if (e.attribute("exterior") == "yes")
+        FP.BackgroundExterior = true;
     if (e.hasAttribute("touchupColor"))
     {
         FP.touchup(
@@ -230,7 +242,9 @@ Painter Painter::fromXML(const QDomElement& e, QString filename)
         FP.setIcon(iconFilename,e.attribute("iconScale", "0.0").toDouble(),e.attribute("iconOffset", "0.0").toDouble());
     }
     if (e.attribute("drawTrafficDirectionMarks") == "yes")
-        FP.drawTrafficDirectionMarks();
+        FP.drawTrafficDirectionMarks(true);
+    if (e.hasAttribute("trafficDirectionMarksColor"))
+        FP.TrafficDirectionMarksColor = toColor((e.attribute("trafficDirectionMarksColor")));
     if (e.hasAttribute("labelColor"))
     {
         FP.label(
@@ -301,11 +315,11 @@ QString Painter::userName() const
     return "Unnamed";
 }
 
-QPair<double, double> Painter::zoomBoundaries() const
+QPair<qreal, qreal> Painter::zoomBoundaries() const
 {
     if (ZoomLimitSet)
         return qMakePair(ZoomUnder,ZoomUpper);
-    return qMakePair(0.0,0.0);
+    return qMakePair((qreal)0.0,(qreal)0.0);
 }
 
 QColor Painter::fillColor() const
@@ -320,13 +334,13 @@ bool Painter::isFilled() const
     return ForegroundFill;
 }
 
-Painter& Painter::drawTrafficDirectionMarks()
+Painter& Painter::drawTrafficDirectionMarks(bool b)
 {
-    DrawTrafficDirectionMarks = true;
+    DrawTrafficDirectionMarks = b;
     return *this;
 }
 
-Painter& Painter::zoomBoundary(double anUnder, double anUpper)
+Painter& Painter::zoomBoundary(qreal anUnder, qreal anUpper)
 {
     ZoomLimitSet = true;
     ZoomUnder = anUnder;
@@ -355,7 +369,7 @@ Painter& Painter::backgroundActive(bool b)
     return *this;
 }
 
-Painter& Painter::background(QColor Color, double Scale, double Offset)
+Painter& Painter::background(QColor Color, qreal Scale, qreal Offset)
 {
     DrawBackground = true;
     BackgroundColor = Color;
@@ -382,7 +396,7 @@ Painter& Painter::touchupActive(bool b)
     return *this;
 }
 
-Painter& Painter::touchupDash(double Dash, double White)
+Painter& Painter::touchupDash(qreal Dash, qreal White)
 {
     TouchupDashSet = true;
     TouchupDash = Dash;
@@ -390,7 +404,7 @@ Painter& Painter::touchupDash(double Dash, double White)
     return *this;
 }
 
-Painter& Painter::touchup(QColor Color, double Scale, double Offset)
+Painter& Painter::touchup(QColor Color, qreal Scale, qreal Offset)
 {
     DrawTouchup = true;
     TouchupColor = Color;
@@ -406,7 +420,7 @@ Painter& Painter::foregroundActive(bool b)
     return *this;
 }
 
-Painter& Painter::foregroundDash(double Dash, double White)
+Painter& Painter::foregroundDash(qreal Dash, qreal White)
 {
     ForegroundDashSet = true;
     ForegroundDash = Dash;
@@ -414,7 +428,7 @@ Painter& Painter::foregroundDash(double Dash, double White)
     return *this;
 }
 
-Painter& Painter::foreground(QColor Color, double Scale, double Offset)
+Painter& Painter::foreground(QColor Color, qreal Scale, qreal Offset)
 {
     DrawForeground = true;
     ForegroundColor = Color;
@@ -465,7 +479,7 @@ Painter& Painter::labelBackgroundTag(const QString& val)
     return *this;
 }
 
-Painter& Painter::label(QColor Color, double Scale, double Offset)
+Painter& Painter::label(QColor Color, qreal Scale, qreal Offset)
 {
     DrawLabel = true;
     LabelColor = Color;
@@ -527,6 +541,16 @@ QString Painter::getLabelBackgroundTag() const
     return LabelBackgroundTag;
 }
 
+bool Painter::getBackgroundInterior() const
+{
+    return BackgroundInterior;
+}
+
+bool Painter::getBackgroundExterior() const
+{
+    return BackgroundExterior;
+}
+
 LineParameters Painter::foregroundBoundary() const
 {
     LineParameters P;
@@ -575,7 +599,7 @@ LineParameters Painter::labelBoundary() const
     return P;
 }
 
-Painter& Painter::setIcon(const QString& Name, double Scale, double Offset)
+Painter& Painter::setIcon(const QString& Name, qreal Scale, qreal Offset)
 {
     DrawIcon = true;
     IconName = Name;
@@ -605,7 +629,7 @@ void Painter::setSelector(const QString& anExpression)
     theSelector = anExpression;
 }
 
-bool Painter::matchesZoom(double PixelPerM) const
+bool Painter::matchesZoom(qreal PixelPerM) const
 {
     if (ZoomLimitSet)
         return (ZoomUnder <= PixelPerM) && (PixelPerM <= ZoomUpper);
@@ -615,12 +639,13 @@ bool Painter::matchesZoom(double PixelPerM) const
 /* GlobalPainter */
 
 GlobalPainter::GlobalPainter()
-: DrawBackground(false)
+    : DrawBackground(false), DrawNodes(false)
 {
 }
 
 GlobalPainter::GlobalPainter(const GlobalPainter& f)
-: DrawBackground(f.DrawBackground), BackgroundColor(f.BackgroundColor)
+    : DrawBackground(f.DrawBackground), BackgroundColor(f.BackgroundColor)
+    , DrawNodes(f.DrawNodes), NodesColor(f.NodesColor), NodesProportional(f.NodesProportional), NodesFixed(f.NodesFixed)
 {
 }
 
@@ -631,6 +656,11 @@ GlobalPainter& GlobalPainter::operator=(const GlobalPainter& f)
     DrawBackground = f.DrawBackground;
     BackgroundColor = f.BackgroundColor;
 
+    DrawNodes = f.DrawNodes;
+    NodesColor = f.NodesColor;
+    NodesProportional = f.NodesProportional;
+    NodesFixed = f.NodesFixed;
+
     return *this;
 }
 
@@ -644,6 +674,8 @@ QString GlobalPainter::toXML() const
     r += "<global\n";
     if (DrawBackground)
         r += " " + colorAsXML("background",BackgroundColor);
+    if (DrawNodes)
+        r += " " + boundaryAsXML("nodes",NodesColor, NodesProportional, NodesFixed);
     r += "/>\n";
     return r;
 }
@@ -656,6 +688,12 @@ GlobalPainter GlobalPainter::fromXML(const QDomElement& e)
         FP.backgroundActive(true);
         FP.background(toColor(e.attribute("backgroundColor")));
     }
+    if (e.hasAttribute("nodesColor")) {
+        FP.nodesActive(true);
+        FP.NodesColor = toColor(e.attribute("nodesColor"));
+        FP.NodesProportional = e.attribute("nodesScale").toDouble();
+        FP.NodesFixed = e.attribute("nodesOffset").toDouble();
+    }
 
     return FP;
 }
@@ -683,3 +721,25 @@ QColor GlobalPainter::getBackgroundColor() const
     return BackgroundColor;
 }
 
+GlobalPainter & GlobalPainter::nodesActive(bool b)
+{
+    DrawNodes = b;
+    return *this;
+}
+
+GlobalPainter & GlobalPainter::nodes(QColor Color)
+{
+    DrawNodes = true;
+    NodesColor = Color;
+    return *this;
+}
+
+bool GlobalPainter::getDrawNodes() const
+{
+    return DrawNodes;
+}
+
+QColor GlobalPainter::getNodesColor() const
+{
+    return NodesColor;
+}
