@@ -30,6 +30,11 @@
 #include "MasPaintStyle.h"
 
 
+// TODO: Replace 'g_Merk_Ignore_Preferences' by having two implementations
+// of the "preferences API": one that behaves as if
+// g_Merk_Ignore_Preferences == false, and one that ignores writes
+// and always returns default settings.
+
 #define M_PARAM_IMPLEMENT_BOOL(Param, Category, Default) \
     bool mb_##Param = false; \
     void MerkaartorPreferences::set##Param(bool theValue) \
@@ -217,31 +222,37 @@ Tool::Tool()
 
 /* MekaartorPreferences */
 
+namespace {
+
+QSettings* getSettings() {
+    if (!g_Merk_Portable) {
+        return new QSettings();
+    } else {
+        return new QSettings(qApp->applicationDirPath() + "/merkaartor.ini", QSettings::IniFormat);
+    }
+}
+
+}  // namespace
+
 MerkaartorPreferences::MerkaartorPreferences()
-    : Sets(0)
 {
     if (!g_Merk_Ignore_Preferences) {
-        if (!g_Merk_Portable) {
-            Sets = new QSettings();
+        Sets = getSettings();
 
-            QSettings oldSettings("BartVanhauwaert", "Merkaartor");
-            QStringList oldKeys = oldSettings.allKeys();
-            foreach(QString k, oldKeys) {
-                Sets->setValue(k, oldSettings.value(k));
-                Sets->sync();
-                oldSettings.remove(k);
-            }
-            oldSettings.clear();
-        } else {
-            Sets = new QSettings(qApp->applicationDirPath() + "/merkaartor.ini", QSettings::IniFormat);
+        QSettings oldSettings("BartVanhauwaert", "Merkaartor");
+        QStringList oldKeys = oldSettings.allKeys();
+        foreach(QString k, oldKeys) {
+            Sets->setValue(k, oldSettings.value(k));
+            Sets->sync();
+            oldSettings.remove(k);
         }
+        oldSettings.clear();
         version = Sets->value("version/version", "0").toString();
     }
 
-    theToolList = new ToolList();
-
-    connect(&httpRequest, SIGNAL(responseHeaderReceived(const QHttpResponseHeader &)), this, SLOT(on_responseHeaderReceived(const QHttpResponseHeader &)));
-    connect(&httpRequest,SIGNAL(requestFinished(int, bool)),this,SLOT(on_requestFinished(int, bool)));
+    connect(&httpRequest, SIGNAL(authenticationRequired(QNetworkReply*, QAuthenticator*)), this, SLOT(on_authenticationRequired(QNetworkReply*, QAuthenticator*)));
+    connect(&httpRequest, SIGNAL(finished(QNetworkReply*)),this,SLOT(on_requestFinished(QNetworkReply*)));
+    connect(&httpRequest, SIGNAL(sslErrors(QNetworkReply *, const QList<QSslError>&)), this, SLOT(on_sslErrors(QNetworkReply*, const QList<QSslError>&)));
 
 #ifdef USE_LIBPROXY
     // Initialise libproxy
@@ -259,7 +270,6 @@ MerkaartorPreferences::MerkaartorPreferences()
 
 MerkaartorPreferences::~MerkaartorPreferences()
 {
-    delete theToolList;
     delete Sets;
 #ifdef USE_LIBPROXY
     px_proxy_factory_free(proxyFactory);
@@ -272,8 +282,6 @@ void MerkaartorPreferences::save(bool UserPwdChanged)
         return;
 
     Sets->setValue("version/version", QString("%1").arg(STRINGIFY(VERSION)));
-    setWmsServers();
-    setTmsServers();
     setTools();
     setAlphaList();
 
@@ -287,6 +295,9 @@ void MerkaartorPreferences::save(bool UserPwdChanged)
     saveTagListFirstColumnWidth();
     Sets->sync();
 
+    // TODO: There is either some misnaming here or a bug. Why would settings
+    // be pulled from OSM only if the password changed, and pushed to OSM
+    // only otherwise?
     if (UserPwdChanged)
         fromOsmPref();
     else
@@ -316,6 +327,7 @@ void MerkaartorPreferences::toOsmPref()
     QByteArray ba = qCompress(theXmlDoc.toString().toUtf8());
     QByteArray PrefsXML = ba.toBase64();
 
+    // TODO: Why is it required to load from PrefsXML in chunk of 254?
     QStringList slicedPrefs;
     for (int i=0; i<PrefsXML.size(); i+=254) {
         QString s = PrefsXML.mid(i, 254);
@@ -348,38 +360,67 @@ void MerkaartorPreferences::fromOsmPref()
 {
     if (getOfflineMode()) return;
 
+    qDebug() << "Requesting preferences from OSM server.";
+
     if (getOsmUser().isEmpty() || getOsmPassword().isEmpty()) return;
 
-    QUrl osmWeb(getOsmApiUrl());
-    if (osmWeb.port() == -1)
-        osmWeb.setPort(80);
+    QUrl osmWeb(getOsmApiUrl()+"/user/preferences");
 
-    httpRequest.setHost(osmWeb.host(), osmWeb.port());
-
-    QHttpRequestHeader Header("GET", osmWeb.path() + "/user/preferences/");
-    if (osmWeb.port() == 80)
-        Header.setValue("Host",osmWeb.host());
-    else
-        Header.setValue("Host",osmWeb.host() + ':' + QString::number(osmWeb.port()));
-    Header.setValue("User-Agent", USER_AGENT);
-
-    QString auth = QString("%1:%2").arg(getOsmUser()).arg(getOsmPassword());
-    QByteArray ba_auth = auth.toUtf8().toBase64();
-    Header.setValue("Authorization", QString("Basic %1").arg(QString(ba_auth)));
+    QNetworkRequest req(osmWeb);
+    req.setRawHeader(QByteArray("User-Agent"), USER_AGENT.toLatin1());
 
     httpRequest.setProxy(getProxy(osmWeb));
-    OsmPrefLoadId = httpRequest.request(Header, NULL, &OsmPrefContent);
+    OsmPrefLoadReply = httpRequest.get(req);
 }
 
-void MerkaartorPreferences::on_requestFinished ( int id, bool error )
+void MerkaartorPreferences::on_authenticationRequired( QNetworkReply *reply, QAuthenticator *auth ) {
+    static QNetworkReply *lastReply = NULL;
+
+    /* Only provide authentication the first time we see this reply, to avoid
+     * infinite loop providing the same credentials. */
+    if (lastReply != reply) {
+        lastReply = reply;
+        qDebug() << "Authentication required and provided.";
+        auth->setUser(getOsmUser());
+        auth->setPassword(getOsmPassword());
+    }
+}
+
+void MerkaartorPreferences::on_sslErrors(QNetworkReply *reply, const QList<QSslError>& errors) {
+    qDebug() << "We stumbled upon some SSL errors: ";
+    foreach ( QSslError error, errors ) {
+        qDebug() << "1:";
+        qDebug() << error.errorString();
+    }
+}
+
+void MerkaartorPreferences::on_requestFinished ( QNetworkReply *reply )
 {
-    if (id != OsmPrefLoadId || error)
+    int error = reply->error();
+    if (error != QNetworkReply::NoError) {
+        qDebug() << "Received response with code " << error << "(" << reply->errorString() << ")";
+        switch (error) {
+            case 406:
+                QMessageBox::critical(NULL,QApplication::translate("MerkaartorPreferences","Preferences upload failed"), QApplication::translate("MerkaartorPreferences","Duplicate key"));
+                return;
+            case 413:
+                QMessageBox::critical(NULL,QApplication::translate("MerkaartorPreferences","Preferences upload failed"), QApplication::translate("MerkaartorPreferences","More than 150 preferences"));
+                return;
+            default:
+                QMessageBox::critical(NULL,QApplication::translate("MerkaartorPreferences","Preferences communication failed"), QApplication::translate("MerkaartorPreferences", "Communication error")+":\n"+reply->errorString());
+                return;
+        }
+    } else {
+        qDebug() << "Received response.";
+    }
+
+    if (reply != OsmPrefLoadReply)
         return;
 
-    QMap<QString, QString> OsmPref;
+    qDebug() << "Reading preferences.";
 
     QDomDocument aOsmPrefDoc;
-    aOsmPrefDoc.setContent(OsmPrefContent.buffer(), false);
+    aOsmPrefDoc.setContent(reply, false);
 
     QDomNodeList prefList = aOsmPrefDoc.elementsByTagName("preference");
 
@@ -410,7 +451,7 @@ void MerkaartorPreferences::on_requestFinished ( int id, bool error )
 
     QByteArray PrefsXML;
     for (int i=0; i<sz; i++)
-        PrefsXML.append(slicedPrefs[i].toAscii());
+        PrefsXML.append(slicedPrefs[i].toLatin1());
 
     //qDebug() << "Size: " << PrefsXML.size();
 
@@ -454,81 +495,43 @@ void MerkaartorPreferences::on_requestFinished ( int id, bool error )
 
         c = c.nextSiblingElement();
     }
+
+    reply->deleteLater();
 }
 
 
 void MerkaartorPreferences::putOsmPref(const QString& k, const QString& v)
 {
-    QUrl osmWeb(getOsmApiUrl());
-    if (osmWeb.port() == -1)
-        osmWeb.setPort(80);
+    qDebug() << "Saving OSM preference online: " << k << "=" << v;
+    QUrl osmWeb(getOsmApiUrl()+QString("/user/preferences/%1").arg(k));
 
     QByteArray ba(v.toUtf8());
     QBuffer Buf(&ba);
 
-    httpRequest.setHost(osmWeb.host(), osmWeb.port());
-
-    QHttpRequestHeader Header("PUT", osmWeb.path() + QString("/user/preferences/%1").arg(k));
-    if (osmWeb.port() == 80)
-        Header.setValue("Host",osmWeb.host());
-    else
-        Header.setValue("Host",osmWeb.host() + ':' + QString::number(osmWeb.port()));
-    Header.setValue("User-Agent", USER_AGENT);
-
-    QString auth = QString("%1:%2").arg(getOsmUser()).arg(getOsmPassword());
-    QByteArray ba_auth = auth.toUtf8().toBase64();
-    Header.setValue("Authorization", QString("Basic %1").arg(QString(ba_auth)));
+    QNetworkRequest req(osmWeb);
 
     httpRequest.setProxy(getProxy(osmWeb));
-    OsmPrefSaveId = httpRequest.request(Header,ba);
+    OsmPrefSaveReply = httpRequest.put(req, ba);
 }
 
 void MerkaartorPreferences::deleteOsmPref(const QString& k)
 {
-    QUrl osmWeb(getOsmApiUrl());
-    if (osmWeb.port() == -1)
-        osmWeb.setPort(80);
+    qDebug() << "Deleting OSM preference online: " << k;
 
-    httpRequest.setHost(osmWeb.host(), osmWeb.port());
+    QUrl osmWeb(getOsmApiUrl()+QString("/user/preferences/%1").arg(k));
 
-    QHttpRequestHeader Header("DELETE", osmWeb.path() + QString("/user/preferences/%1").arg(k));
-    if (osmWeb.port() == 80)
-        Header.setValue("Host",osmWeb.host());
-    else
-        Header.setValue("Host",osmWeb.host() + ':' + QString::number(osmWeb.port()));
-    Header.setValue("User-Agent", USER_AGENT);
-
-    QString auth = QString("%1:%2").arg(getOsmUser()).arg(getOsmPassword());
-    QByteArray ba_auth = auth.toUtf8().toBase64();
-    Header.setValue("Authorization", QString("Basic %1").arg(QString(ba_auth)));
+    QNetworkRequest req(osmWeb);
 
     httpRequest.setProxy(getProxy(osmWeb));
-    httpRequest.request(Header);
-}
-
-void MerkaartorPreferences::on_responseHeaderReceived(const QHttpResponseHeader & hdr)
-{
-    switch (hdr.statusCode()) {
-        case 200:
-            break;
-        case 406:
-            QMessageBox::critical(NULL,QApplication::translate("MerkaartorPreferences","Preferences upload failed"), QApplication::translate("MerkaartorPreferences","Duplicate key"));
-            break;
-        case 413:
-            QMessageBox::critical(NULL,QApplication::translate("MerkaartorPreferences","Preferences upload failed"), QApplication::translate("MerkaartorPreferences","More than 150 preferences"));
-            break;
-        default:
-            qDebug() << "MerkaartorPreferences::on_responseHeaderReceived :" << hdr.statusCode() << hdr.reasonPhrase();
-            break;
-    }
+    OsmPrefSaveReply = httpRequest.sendCustomRequest(req,"DELETE");
 }
 
 void MerkaartorPreferences::initialize()
 {
-//	Use06Api = Sets->value("osm/use06api", "true").toBool();
+//  Use06Api = Sets->value("osm/use06api", "true").toBool();
     Use06Api = true;
 
-    // PRoxy upgrade
+    // Proxy upgrade
     if (!g_Merk_Ignore_Preferences && !g_Merk_Reset_Preferences) {
         if (Sets->contains("proxy/Use")) {
             bool b = Sets->value("proxy/Use").toBool();
@@ -582,17 +585,20 @@ void MerkaartorPreferences::initialize()
         tl = Sets->value("Tools/list").toStringList();
         for (int i=0; i<tl.size(); i+=TOOL_FIELD_SIZE) {
             Tool t(tl[i], tl[i+1]);
-            theToolList->insert(tl[i], t);
+            theToolList.insert(tl[i], t);
         }
     }
-    if (!theToolList->contains("Inkscape")) {
+    if (!theToolList.contains("Inkscape")) {
         Tool t("Inkscape", "");
-        theToolList->insert("Inkscape", t);
+        theToolList.insert("Inkscape", t);
     }
 
     QStringList Servers;
     if (!g_Merk_Ignore_Preferences && !g_Merk_Reset_Preferences) {
         Servers = Sets->value("WSM/servers").toStringList();
+	// TODO: Apparently WMS/servers is a list, and every 7 consecutive
+	// items describe a single server. There should be some documentation
+	// about what do the fields mean. Same with TMS/servers.
         if (Servers.size()) {
             for (int i=0; i<Servers.size(); i+=7) {
                 WmsServer S(Servers[i], Servers[i+1], Servers[i+2], Servers[i+3], Servers[i+4], Servers[i+5], Servers[i+6], "", "");
@@ -663,28 +669,10 @@ BookmarkList* MerkaartorPreferences::getBookmarks()
 
 WmsServerList* MerkaartorPreferences::getWmsServers()
 {
-//	return Sets->value("WSM/servers").toStringList();
+//  return Sets->value("WSM/servers").toStringList();
     return theWmsServerList.getServers();
 }
 
-void MerkaartorPreferences::setWmsServers()
-{
-    //QStringList Servers;
-    //WmsServerListIterator i(theWmsServerList);
-    //while (i.hasNext()) {
-    //	i.next();
-    //	WmsServer S = i.value();
-    //	Servers.append(S.WmsName);
-    //	Servers.append(S.WmsAdress);
-    //	Servers.append(S.WmsPath);
-    //	Servers.append(S.WmsLayers);
-    //	Servers.append(S.WmsProjections);
-    //	Servers.append(S.WmsStyles);
-    //	Servers.append(S.WmsImgFormat);
-    //}
-    //Sets->setValue("WSM/servers", Servers);
-}
-
 OsmServerList* MerkaartorPreferences::getOsmServers()
 {
     return &theOsmServers;
@@ -694,27 +682,10 @@ OsmServerList* MerkaartorPreferences::getOsmServers()
 
 TmsServerList* MerkaartorPreferences::getTmsServers()
 {
-//	return Sets->value("WSM/servers").toStringList();
+//  return Sets->value("WSM/servers").toStringList();
     return theTmsServerList.getServers();
 }
 
-void MerkaartorPreferences::setTmsServers()
-{
-    //QStringList Servers;
-    //TmsServerListIterator i(theTmsServerList);
-    //while (i.hasNext()) {
-    //	i.next();
-    //	TmsServer S = i.value();
-    //	Servers.append(S.TmsName);
-    //	Servers.append(S.TmsAdress);
-    //	Servers.append(S.TmsPath);
-    //	Servers.append(QString::number(S.TmsTileSize));
-    //	Servers.append(QString::number(S.TmsMinZoom));
-    //	Servers.append(QString::number(S.TmsMaxZoom));
-    //}
-    //Sets->setValue("TMS/servers", Servers);
-}
-
 /* */
 
 M_PARAM_IMPLEMENT_STRING(SelectedServer, backgroundImage, "");
@@ -892,12 +863,6 @@ ProjectionItem MerkaartorPreferences::getProjection(QString aProj)
 }
 #endif
 
-void MerkaartorPreferences::setCurrentFilter(FilterType theValue)
-{
-    if (!g_Merk_Ignore_Preferences)
-        Sets->setValue("filter/Type", theValue);
-}
-
 QString MerkaartorPreferences::getCurrentFilter()
 {
     if (!g_Merk_Ignore_Preferences && !g_Merk_Reset_Preferences)
@@ -1102,16 +1067,16 @@ ExportType MerkaartorPreferences::getExportType() const
 }
 
 /* Tools */
-ToolList* MerkaartorPreferences::getTools() const
+ToolList* MerkaartorPreferences::getTools()
 {
-    return theToolList;
+    return &theToolList;
 }
 
 void MerkaartorPreferences::setTools()
 {
     if (!g_Merk_Ignore_Preferences) {
         QStringList tl;
-        ToolListIterator i(*theToolList);
+        ToolListIterator i(theToolList);
         while (i.hasNext()) {
             i.next();
             Tool t = i.value();
@@ -1126,7 +1091,7 @@ Tool MerkaartorPreferences::getTool(QString toolName) const
 {
     Tool ret;
 
-    ToolListIterator i(*theToolList);
+    ToolListIterator i(theToolList);
     while (i.hasNext()) {
         i.next();
         if (i.key() == toolName) {
@@ -1377,55 +1342,74 @@ M_PARAM_IMPLEMENT_BOOL(ReadonlyTracksDefault, data, false)
 /* FeaturesDock */
 M_PARAM_IMPLEMENT_BOOL(FeaturesWithin, FeaturesDock, true)
 
-/* Projections */
-void MerkaartorPreferences::loadProjection(QString fn)
-{
-    if (QDir::isRelativePath(fn))
-        fn = QCoreApplication::applicationDirPath() + "/" + fn;
-
-    qDebug() << "loadProjection " << fn;
-    QFile file(fn);
-    if (!file.open(QIODevice::ReadOnly)) {
-//		QMessageBox::critical(this, tr("Invalid file"), tr("%1 could not be opened.").arg(fn));
-        return;
-    }
-
-    QDomDocument* theXmlDoc = new QDomDocument();
-    if (!theXmlDoc->setContent(&file)) {
-//		QMessageBox::critical(this, tr("Invalid file"), tr("%1 is not a valid XML file.").arg(fn));
-        file.close();
-        return;
-    }
-    file.close();
-
-    QDomElement docElem = theXmlDoc->documentElement();
-    ProjectionsList aProjList = ProjectionsList::fromXml(docElem.firstChildElement());
-    theProjectionsList.add(aProjList);
-
-    delete theXmlDoc;
-}
-
-void MerkaartorPreferences::loadProjections()
-{
-    QString fn;
-
-    fn = HOMEDIR + "/Projections.xml";
-    loadProjection(fn);
+namespace {
 
+// Preference XMLs may be stored in several directories depending
+// on the platform. This method returns the list of directories to load
+// preference XMLs from.
+QStringList getPreferenceDirectories() {
+    QList<QString> directories;
+    directories << HOMEDIR;
+    // TODO: Some files are loaded without this override for Q_OS_MAC. Why?
 #if defined(Q_OS_MAC)
     {
         QDir resources = QDir(QCoreApplication::applicationDirPath());
         resources.cdUp();
         resources.cd("Resources");
-        fn = resources.absolutePath() + "/Projections.xml";
+        directories << resources.absolutePath();
     }
 #else
-    fn = QString(SHAREDIR) + "/Projections.xml";
+    directories << QString(SHAREDIR);
 #endif
-    loadProjection(fn);
+    directories << ":";
+    return directories;
+}
 
-    fn = ":/Projections.xml";
-    loadProjection(fn);
+// Returns the list of all alternative locations of the given preference
+// file.
+QStringList getPreferenceFilePaths(QString fileName) {
+    QList<QString> paths;
+    const QStringList directories = getPreferenceDirectories();
+    for (QStringList::const_iterator i = directories.begin(); i != directories.end(); ++i) {
+	paths << (*i) + "/" + fileName;
+    }
+    return paths;
+}
+
+}  // namespace
+
+/* Projections */
+void MerkaartorPreferences::loadProjectionsFromFile(QString fileName)
+{
+    if (QDir::isRelativePath(fileName))
+        fileName = QCoreApplication::applicationDirPath() + "/" + fileName;
+
+    qDebug() << "loadProjection " << fileName;
+    QFile file(fileName);
+    if (!file.open(QIODevice::ReadOnly)) {
+//      QMessageBox::critical(this, tr("Invalid file"), tr("%1 could not be opened.").arg(fileName));
+        return;
+    }
+
+    QDomDocument theXmlDoc;
+    if (!theXmlDoc.setContent(&file)) {
+//		QMessageBox::critical(this, tr("Invalid file"), tr("%1 is not a valid XML file.").arg(fileName));
+        file.close();
+        return;
+    }
+    file.close();
+
+    QDomElement docElem = theXmlDoc.documentElement();
+    ProjectionsList aProjList = ProjectionsList::fromXml(docElem.firstChildElement());
+    theProjectionsList.add(aProjList);
+}
+
+void MerkaartorPreferences::loadProjections()
+{
+    const QStringList paths = getPreferenceFilePaths("Projections.xml");
+    for (QStringList::const_iterator i = paths.begin(); i != paths.end(); ++i) {
+	loadProjectionsFromFile(*i);
+    }
 }
 
 void MerkaartorPreferences::saveProjections()
@@ -1448,52 +1432,35 @@ void MerkaartorPreferences::saveProjections()
 }
 
 /* Filters */
-void MerkaartorPreferences::loadFilter(QString fn)
+void MerkaartorPreferences::loadFiltersFromFile(QString fileName)
 {
-    if (QDir::isRelativePath(fn))
-        fn = QCoreApplication::applicationDirPath() + "/" + fn;
+    if (QDir::isRelativePath(fileName))
+        fileName = QCoreApplication::applicationDirPath() + "/" + fileName;
 
-    qDebug() << "loadFilter " << fn;
-    QFile file(fn);
+    qDebug() << "loadFiltersFromFile " << fileName;
+    QFile file(fileName);
     if (!file.open(QIODevice::ReadOnly)) {
         return;
     }
 
-    QDomDocument* theXmlDoc = new QDomDocument();
-    if (!theXmlDoc->setContent(&file)) {
+    QDomDocument theXmlDoc;
+    if (!theXmlDoc.setContent(&file)) {
         file.close();
         return;
     }
     file.close();
 
-    QDomElement docElem = theXmlDoc->documentElement();
+    QDomElement docElem = theXmlDoc.documentElement();
     FiltersList aFilterList = FiltersList::fromXml(docElem.firstChildElement());
     theFiltersList.add(aFilterList);
-
-    delete theXmlDoc;
 }
 
 void MerkaartorPreferences::loadFilters()
 {
-    QString fn;
-
-    fn = HOMEDIR + "/Filters.xml";
-    loadFilter(fn);
-
-#if defined(Q_OS_MAC)
-    {
-        QDir resources = QDir(QCoreApplication::applicationDirPath());
-        resources.cdUp();
-        resources.cd("Resources");
-        fn = resources.absolutePath() + "/Filters.xml";
+    const QStringList paths = getPreferenceFilePaths("Filters.xml");
+    for (QStringList::const_iterator i = paths.begin(); i != paths.end(); ++i) {
+        loadFiltersFromFile(*i);
     }
-#else
-    fn = QString(SHAREDIR) + "/Filters.xml";
-#endif
-    loadFilter(fn);
-
-    fn = ":/Filters.xml";
-    loadFilter(fn);
 }
 
 void MerkaartorPreferences::saveFilters()
@@ -1516,44 +1483,37 @@ void MerkaartorPreferences::saveFilters()
 
 
 /* WMS Servers */
-void MerkaartorPreferences::loadWMS(QString fn)
+void MerkaartorPreferences::loadWMSesFromFile(QString fileName)
 {
-    if (QDir::isRelativePath(fn))
-        fn = QCoreApplication::applicationDirPath() + "/" + fn;
+    if (QDir::isRelativePath(fileName))
+        fileName = QCoreApplication::applicationDirPath() + "/" + fileName;
 
-    QFile file(fn);
+    QFile file(fileName);
     if (!file.open(QIODevice::ReadOnly)) {
-//		QMessageBox::critical(this, tr("Invalid file"), tr("%1 could not be opened.").arg(fn));
+//      QMessageBox::critical(this, tr("Invalid file"), tr("%1 could not be opened.").arg(fileName));
         return;
     }
 
-    QDomDocument* theXmlDoc = new QDomDocument();
-    if (!theXmlDoc->setContent(&file)) {
-//		QMessageBox::critical(this, tr("Invalid file"), tr("%1 is not a valid XML file.").arg(fn));
+    QDomDocument theXmlDoc;
+    if (!theXmlDoc.setContent(&file)) {
+//		QMessageBox::critical(this, tr("Invalid file"), tr("%1 is not a valid XML file.").arg(fileName));
         file.close();
         return;
     }
     file.close();
 
-    QDomElement docElem = theXmlDoc->documentElement();
+    QDomElement docElem = theXmlDoc.documentElement();
     WmsServersList aWmsList = WmsServersList::fromXml(docElem.firstChildElement());
     theWmsServerList.add(aWmsList);
-
-    delete theXmlDoc;
 }
 
 void MerkaartorPreferences::loadWMSes()
 {
-    QString fn;
-
-    fn = HOMEDIR + "/WmsServersList.xml";
-    loadWMS(fn);
-
-    fn = QString(SHAREDIR) + "/WmsServersList.xml";
-    loadWMS(fn);
-
-    fn = ":/WmsServersList.xml";
-    loadWMS(fn);
+    loadWMSesFromFile(HOMEDIR + "/WmsServersList.xml");
+    // TODO: Why is the Q_OS_MAC override in getPreferenceDirectories()
+    // missing here? Is that a bug, or an intention?
+    loadWMSesFromFile(QString(SHAREDIR) + "/WmsServersList.xml");
+    loadWMSesFromFile(":/WmsServersList.xml");
 }
 
 void MerkaartorPreferences::saveWMSes()
@@ -1576,44 +1536,37 @@ void MerkaartorPreferences::saveWMSes()
 }
 
 /* TMS Servers */
-void MerkaartorPreferences::loadTMS(QString fn)
+void MerkaartorPreferences::loadTMSesFromFile(QString fileName)
 {
-    if (QDir::isRelativePath(fn))
-        fn = QCoreApplication::applicationDirPath() + "/" + fn;
+    if (QDir::isRelativePath(fileName))
+        fileName = QCoreApplication::applicationDirPath() + "/" + fileName;
 
-    QFile file(fn);
+    QFile file(fileName);
     if (!file.open(QIODevice::ReadOnly)) {
-//		QMessageBox::critical(this, tr("Invalid file"), tr("%1 could not be opened.").arg(fn));
+//      QMessageBox::critical(this, tr("Invalid file"), tr("%1 could not be opened.").arg(fileName));
         return;
     }
 
-    QDomDocument* theXmlDoc = new QDomDocument();
-    if (!theXmlDoc->setContent(&file)) {
-//		QMessageBox::critical(this, tr("Invalid file"), tr("%1 is not a valid XML file.").arg(fn));
+    QDomDocument theXmlDoc;
+    if (!theXmlDoc.setContent(&file)) {
+//		QMessageBox::critical(this, tr("Invalid file"), tr("%1 is not a valid XML file.").arg(fileName));
         file.close();
         return;
     }
     file.close();
 
-    QDomElement docElem = theXmlDoc->documentElement();
+    QDomElement docElem = theXmlDoc.documentElement();
     TmsServersList aTmsList = TmsServersList::fromXml(docElem.firstChildElement());
     theTmsServerList.add(aTmsList);
-
-    delete theXmlDoc;
 }
 
 void MerkaartorPreferences::loadTMSes()
 {
-    QString fn;
-
-    fn = HOMEDIR + "/TmsServersList.xml";
-    loadTMS(fn);
-
-    fn = QString(SHAREDIR) + "/TmsServersList.xml";
-    loadTMS(fn);
-
-    fn = ":/TmsServersList.xml";
-    loadTMS(fn);
+    loadTMSesFromFile(HOMEDIR + "/TmsServersList.xml");
+    // TODO: Why is the Q_OS_MAC override in getPreferenceDirectories()
+    // missing here? Is that a bug, or an intention?
+    loadTMSesFromFile(QString(SHAREDIR) + "/TmsServersList.xml");
+    loadTMSesFromFile(":/TmsServersList.xml");
 }
 
 void MerkaartorPreferences::saveTMSes()
@@ -1636,44 +1589,37 @@ void MerkaartorPreferences::saveTMSes()
 }
 
 /* Bookmarks */
-void MerkaartorPreferences::loadBookmark(QString fn)
+void MerkaartorPreferences::loadBookmarksFromFile(QString fileName)
 {
-    if (QDir::isRelativePath(fn))
-        fn = QCoreApplication::applicationDirPath() + "/" + fn;
+    if (QDir::isRelativePath(fileName))
+        fileName = QCoreApplication::applicationDirPath() + "/" + fileName;
 
-    QFile file(fn);
+    QFile file(fileName);
     if (!file.open(QIODevice::ReadOnly)) {
-//		QMessageBox::critical(this, tr("Invalid file"), tr("%1 could not be opened.").arg(fn));
+//      QMessageBox::critical(this, tr("Invalid file"), tr("%1 could not be opened.").arg(fileName));
         return;
     }
 
-    QDomDocument* theXmlDoc = new QDomDocument();
-    if (!theXmlDoc->setContent(&file)) {
-//		QMessageBox::critical(this, tr("Invalid file"), tr("%1 is not a valid XML file.").arg(fn));
+    QDomDocument theXmlDoc;
+    if (!theXmlDoc.setContent(&file)) {
+//		QMessageBox::critical(this, tr("Invalid file"), tr("%1 is not a valid XML file.").arg(fileName));
         file.close();
         return;
     }
     file.close();
 
-    QDomElement docElem = theXmlDoc->documentElement();
+    QDomElement docElem = theXmlDoc.documentElement();
     BookmarksList aBkList = BookmarksList::fromXml(docElem.firstChildElement());
     theBookmarkList.add(aBkList);
-
-    delete theXmlDoc;
 }
 
 void MerkaartorPreferences::loadBookmarks()
 {
-    QString fn;
-
-    fn = HOMEDIR + "/BookmarksList.xml";
-    loadBookmark(fn);
-
-    fn = QString(SHAREDIR) + "/BookmarksList.xml";
-    loadBookmark(fn);
-
-    fn = ":/BookmarksList.xml";
-    loadBookmark(fn);
+    loadBookmarksFromFile(HOMEDIR + "/BookmarksList.xml");
+    // TODO: Why is the Q_OS_MAC override in getPreferenceDirectories()
+    // missing here? Is that a bug, or an intention?
+    loadBookmarksFromFile(QString(SHAREDIR) + "/BookmarksList.xml");
+    loadBookmarksFromFile(":/BookmarksList.xml");
 }
 
 void MerkaartorPreferences::saveBookmarks()
@@ -1735,14 +1681,9 @@ void MerkaartorPreferences::saveOsmServers()
 QString getDefaultLanguage(bool returnDefault)
 {
     if (!g_Merk_Ignore_Preferences && !g_Merk_Reset_Preferences) {
-        QSettings* Sets;
-        if (!g_Merk_Portable) {
-            Sets = new QSettings();
-        } else {
-            Sets = new QSettings(qApp->applicationDirPath() + "/merkaartor.ini", QSettings::IniFormat);
-        }
-        QString lang = Sets->value("locale/language").toString();
-        delete Sets;
+        QSettings* sets = getSettings();
+        QString lang = sets->value("locale/language").toString();
+        delete sets;
         if (lang == "")
             if (returnDefault)
                 lang = QLocale::system().name().split("_")[0];
@@ -1758,12 +1699,8 @@ QString getDefaultLanguage(bool returnDefault)
 void setDefaultLanguage(const QString& theValue)
 {
     if (!g_Merk_Ignore_Preferences) {
-        QSettings* Sets;
-        if (!g_Merk_Portable) {
-            Sets = new QSettings();
-        } else {
-            Sets = new QSettings(qApp->applicationDirPath() + "/merkaartor.ini", QSettings::IniFormat);
-        }
-        Sets->setValue("locale/language", theValue);
+        QSettings* sets = getSettings();
+        sets->setValue("locale/language", theValue);
+        // TODO: 'sets' memory leak?
     }
 }
