@@ -514,13 +514,17 @@ void PreferencesDialog::on_BrowseTemplate_clicked()
         CustomTemplateName->setText(QDir::toNativeSeparators(s));
 }
 
-void PreferencesDialog::on_btBgColor_clicked()
-{
-    QColor rgb = QColorDialog::getColor(BgColor, this
+QColor PreferencesDialog::pickColor(QColor defaultColor) {
+    return QColorDialog::getColor(defaultColor, this
 #if (QT_VERSION >= QT_VERSION_CHECK(4, 5, 0))
                                         , tr("Select Color"), QColorDialog::ShowAlphaChannel
 #endif
                                        );
+}
+
+void PreferencesDialog::on_btBgColor_clicked()
+{
+    QColor rgb = pickColor(BgColor);
     if (rgb.isValid()) {
         BgColor = rgb;
         makeBoundaryIcon(btBgColor, BgColor);
@@ -529,11 +533,7 @@ void PreferencesDialog::on_btBgColor_clicked()
 
 void PreferencesDialog::on_btFocusColor_clicked()
 {
-    QColor rgb = QColorDialog::getColor(FocusColor, this
-#if (QT_VERSION >= QT_VERSION_CHECK(4, 5, 0))
-                                        , tr("Select Color"), QColorDialog::ShowAlphaChannel
-#endif
-                                       );
+    QColor rgb = pickColor(FocusColor);
     if (rgb.isValid()) {
         FocusColor = rgb;
         makeBoundaryIcon(btFocusColor, FocusColor);
@@ -542,11 +542,7 @@ void PreferencesDialog::on_btFocusColor_clicked()
 
 void PreferencesDialog::on_btHoverColor_clicked()
 {
-    QColor rgb = QColorDialog::getColor(HoverColor, this
-#if (QT_VERSION >= QT_VERSION_CHECK(4, 5, 0))
-                                        , tr("Select Color"), QColorDialog::ShowAlphaChannel
-#endif
-                                       );
+    QColor rgb = pickColor(HoverColor);
     if (rgb.isValid()) {
         HoverColor = rgb;
         makeBoundaryIcon(btHoverColor, HoverColor);
@@ -555,11 +551,7 @@ void PreferencesDialog::on_btHoverColor_clicked()
 
 void PreferencesDialog::on_btHighlightColor_clicked()
 {
-    QColor rgb = QColorDialog::getColor(HighlightColor, this
-#if (QT_VERSION >= QT_VERSION_CHECK(4, 5, 0))
-                                        , tr("Select Color"), QColorDialog::ShowAlphaChannel
-#endif
-                                       );
+    QColor rgb = pickColor(HighlightColor);
     if (rgb.isValid()) {
         HighlightColor = rgb;
         makeBoundaryIcon(btHighlightColor, HighlightColor);
@@ -568,11 +560,7 @@ void PreferencesDialog::on_btHighlightColor_clicked()
 
 void PreferencesDialog::on_btDirtyColor_clicked()
 {
-    QColor rgb = QColorDialog::getColor(DirtyColor, this
-#if (QT_VERSION >= QT_VERSION_CHECK(4, 5, 0))
-                                        , tr("Select Color"), QColorDialog::ShowAlphaChannel
-#endif
-                                       );
+    QColor rgb = pickColor(DirtyColor);
     if (rgb.isValid()) {
         DirtyColor = rgb;
         makeBoundaryIcon(btDirtyColor, DirtyColor);
@@ -581,11 +569,7 @@ void PreferencesDialog::on_btDirtyColor_clicked()
 
 void PreferencesDialog::on_btRelationsColor_clicked()
 {
-    QColor rgb = QColorDialog::getColor(RelationsColor, this
-#if (QT_VERSION >= QT_VERSION_CHECK(4, 5, 0))
-                                        , tr("Select Color"), QColorDialog::ShowAlphaChannel
-#endif
-                                       );
+    QColor rgb = pickColor(RelationsColor);
     if (rgb.isValid()) {
         RelationsColor = rgb;
         makeBoundaryIcon(btRelationsColor, RelationsColor);
@@ -593,11 +577,7 @@ void PreferencesDialog::on_btRelationsColor_clicked()
 }
 void PreferencesDialog::on_btGpxTrackColor_clicked()
 {
-    QColor rgb = QColorDialog::getColor(GpxTrackColor, this
-#if (QT_VERSION >= QT_VERSION_CHECK(4, 5, 0))
-                                        , tr("Select Color"), QColorDialog::ShowAlphaChannel
-#endif
-                                       );
+    QColor rgb = pickColor(GpxTrackColor);
     if (rgb.isValid()) {
         GpxTrackColor = rgb;
         makeBoundaryIcon(btGpxTrackColor, GpxTrackColor);
