@@ -21,7 +21,7 @@
 #define WMSCMAPADAPTER_H
 
 #include "mapadapter.h"
-#include "Preferences/WmsServersList.h"
+#include "WmsServersList.h"
 
 //! MapAdapter for servers with image tiles
 /*!
@@ -71,6 +71,7 @@ public:
      * @return the source tag
      */
     virtual QString	getSourceTag() const;
+    virtual void setSourceTag (const QString& ) {}
 
     //! returns the Url of the usage license
     /*!
@@ -78,7 +79,7 @@ public:
      */
     virtual QString	getLicenseUrl() const;
 
-    double PI;
+    qreal PI;
 
     virtual void zoom_in();
     virtual void zoom_out();
@@ -97,8 +98,8 @@ public:
 
     virtual void cleanup() {}
 
-    virtual bool toXML(QDomElement /*xParent*/) { return true; }
-    virtual void fromXML(const QDomElement /*xParent*/) {}
+    virtual bool toXML(QXmlStreamWriter& /*stream*/) { return true; }
+    virtual void fromXML(QXmlStreamReader& /*xParent*/) {}
     virtual QString toPropertiesHtml() {return "";}
 
     virtual void setSettings(QSettings* /*aSet*/) {}
