@@ -19,7 +19,7 @@
  ***************************************************************************/
 #include "mapnetwork.h"
 
-#include "Preferences/MerkaartorPreferences.h"
+#include "MerkaartorPreferences.h"
 
 #include <QNetworkRequest>
 #include <QNetworkReply>
@@ -60,7 +60,13 @@ void MapNetwork::launchRequest()
         return;
     LoadingRequest* R = loadingRequests.dequeue();
 
-    QUrl U("http://" + QString(R->host).append(R->url));
+    QUrl U;
+    if (!R->host.contains("://")) {
+        U.setUrl("http://" + QString(R->host).append(R->url));
+    } else {
+        U.setUrl(QString(R->host).append(R->url));
+    }
+
     qDebug() << "getting: " << U.toString();
 
     launchRequest(U, R);
@@ -127,10 +133,15 @@ void MapNetwork::requestFinished(QNetworkReply* reply)
                 QString hash = R->hash;
                 // 		qDebug() << "request finished for id: " << id;
                 QByteArray ax;
+                QHash<QString, QString> headers;
 
                 if (reply->bytesAvailable() > 0) {
                     ax = reply->readAll();
-                    parent->receivedData(ax, hash);
+                    foreach (QByteArray k, reply->rawHeaderList()) {
+                        headers[QString(k)] = QString(reply->rawHeader(k));
+                    }
+
+                    parent->receivedData(ax, headers, hash);
                 }
                 break;
         }
