@@ -71,6 +71,7 @@ public:
      * @return the source tag
      */
     virtual QString	getSourceTag() const;
+    virtual void setSourceTag (const QString& ) {};
 
     //! returns the Url of the usage license
     /*!
@@ -94,8 +95,8 @@ public:
 
     virtual void cleanup() {}
 
-    virtual bool toXML(QDomElement /*xParent*/) { return true; }
-    virtual void fromXML(const QDomElement /*xParent*/) {}
+    virtual bool toXML(QXmlStreamWriter& /*stream*/) { return true; }
+    virtual void fromXML(QXmlStreamReader& /*xParent*/) {}
     virtual QString toPropertiesHtml() {return "";}
 
     virtual void setSettings(QSettings* /*aSet*/) {}
