@@ -28,23 +28,15 @@
 #include <QStatusBar>
 #include <QInputDialog>
 
-#include <zlib.h>
-
-// #define DEBUG_EVERY_CALL
-// #define DEBUG_MAPCALL_ONLY
-// #define DEBUG_NONGET_CALL
-
 /* DOWNLOADER */
 
 Downloader::Downloader(const QString& aUser, const QString& aPwd)
 : User(aUser), Password(aPwd),
-  Id(0),Error(false), AnimatorLabel(0), AnimatorBar(0), AnimationTimer(0)
+  currentReply(0),Error(false), AnimatorLabel(0), AnimatorBar(0), AnimationTimer(0)
 {
     //IdAuth = Request.setUser(User.toUtf8(), Password.toUtf8());
-//	connect(&Request,SIGNAL(done(bool)), this,SLOT(allDone(bool)));
-    connect(&Request,SIGNAL(requestFinished(int, bool)),this,SLOT(on_requestFinished(int, bool)));
-    connect(&Request,SIGNAL(dataReadProgress(int, int)), this,SLOT(progress(int, int)));
-    connect(&Request, SIGNAL(responseHeaderReceived(const QHttpResponseHeader &)), this, SLOT(on_responseHeaderReceived(const QHttpResponseHeader &)));
+    connect(&netManager,SIGNAL(finished(QNetworkReply*)),this,SLOT(on_requestFinished(QNetworkReply*)));
+    connect(&netManager,SIGNAL(authenticationRequired(QNetworkReply*,QAuthenticator*)), this,SLOT(on_authenticationRequired(QNetworkReply*,QAuthenticator*)));
 }
 
 
@@ -82,248 +74,91 @@ void Downloader::on_Cancel_clicked()
         Loop.exit(QDialog::Rejected);
 }
 
-#include "QtGui/QTextBrowser"
+#include "QTextBrowser"
 
-void showDebug(const QString& Method, const QString& URL, const QString& Sent, const QByteArray& arr)
-{
-    static int Download = 0;
-    ++Download;
-    QTextBrowser* b = new QTextBrowser;
-    b->setWindowTitle(Method + " " + URL+" -- "+QString::number(Download));
-    if (Sent.length())
-    {
-        b->append(Sent+QString("\n"));
-        b->append(" <======================== "+QString("\n"));
-        b->append(" ========================> "+QString("\n"));
-    }
-    b->append(QString::fromUtf8(arr));
-    b->setAttribute(Qt::WA_DeleteOnClose,true);
-    b->show();
-    b->raise();
+bool Downloader::go(const QUrl& url) {
+    return request("GET", url, QString(), false);
 }
 
-
-#define CHUNK 4096
-
-QByteArray gzipDecode(const QByteArray& In)
-{
-    QByteArray Total;
-
-    int ret;
-    unsigned have;
-    z_stream strm;
-    char in[CHUNK+2];
-    char out[CHUNK+2];
-    /* allocate inflate state */
-    strm.zalloc = Z_NULL;
-    strm.zfree = Z_NULL;
-    strm.opaque = Z_NULL;
-    strm.avail_in = 0;
-    strm.next_in = Z_NULL;
-    ret = inflateInit2(&strm,15+32);
-    if (ret != Z_OK)
-       {
-               (void)inflateEnd(&strm);
-        return Total;
-       }
-    int RealSize = In.size();
-    for (int i=0; i<RealSize/CHUNK+1; ++i)
-    {
-        int Left = RealSize-(i*CHUNK);
-        if (Left > CHUNK)
-            Left = CHUNK;
-        memcpy(in,In.constData()+(i*CHUNK),Left);
-        strm.avail_in = Left;
-        strm.next_in = reinterpret_cast<unsigned char*>(in);
-
-        /* run inflate() on input until output buffer not full */
-        do
-        {
-            strm.avail_out = CHUNK;
-            strm.next_out = reinterpret_cast<unsigned char*>(out);
-            ret = inflate(&strm, Z_NO_FLUSH);
-            if (ret == Z_STREAM_ERROR)
-                       {
-                               (void)inflateEnd(&strm);
-                return Total;
-                       }
-            switch (ret)
-            {
-                case Z_NEED_DICT:
-                    ret = Z_DATA_ERROR;     /* and fall through */
-                case Z_DATA_ERROR:
-                case Z_MEM_ERROR:
-                    (void)inflateEnd(&strm);
-                    return Total;
-            }
-            have = CHUNK - strm.avail_out;
-            out[have] = 0;
-            Total.append(QByteArray(out,have));
-        } while (strm.avail_out == 0);
-    }
-       (void)inflateEnd(&strm);
-    return Total;
-}
-
-
-
-
-bool Downloader::go(const QUrl& url)
-{
+bool Downloader::request(const QString& theMethod, const QUrl& url, const QString& theData, bool FireForget) {
     if (Error) return false;
 
-    Request.setProxy(M_PREFS->getProxy(url));
-    Request.setHost(url.host(),url.port(80));
-
-    if (AnimationTimer)
-        AnimationTimer->start(200);
-    Content.clear();
-    QBuffer ResponseBuffer(&Content);
-    ResponseBuffer.open(QIODevice::WriteOnly);
-    QString sReq = url.toString(QUrl::RemoveScheme | QUrl::RemoveAuthority);
-    qDebug() << "Downloader::go: " << url << sReq;
-    QHttpRequestHeader Header("GET",sReq);
-    Header.setValue("Accept-Encoding", "gzip,deflate");
-    Header.setValue("Host",url.host()+':'+QString::number(url.port(80)));
-    Header.setValue("User-Agent", USER_AGENT);
-    Content.clear();
-    Id = Request.request(Header,QByteArray(), &ResponseBuffer);
-
-    if (Loop.exec() == QDialog::Rejected)
-    {
-        Request.abort();
-        return false;
-    }
-
-    if (Error)
-    {
-        QMessageBox::information(0,tr("error"),Request.errorString());
-    }
-    if (Request.lastResponse().hasContentLength() && Content.size() != (int)Request.lastResponse().contentLength())
-    {
-        QMessageBox::information(0,tr("didn't download enough"),QString("%1 %2").arg(Content.size()).arg(Request.lastResponse().contentLength()));
-    }
-
-    if (Request.lastResponse().hasKey("Content-encoding"))
-    {
-        QString t(Request.lastResponse().value("Content-encoding"));
-        if (t == "gzip")
-        {
-            QByteArray Uncompressed(gzipDecode(Content));
-            Content = Uncompressed;
-        }
-    }
-
-#ifdef DEBUG_EVERY_CALL
-    showDebug("GET", Web.path() + url, QByteArray() ,Content);
-#endif
-    SAFE_DELETE(AnimationTimer);
-    LocationText = Request.lastResponse().value("Location");
-
-    Result = Request.lastResponse().statusCode();
-    switch (Result) {
-    case 301:
-    case 302:
-    case 307: {
-            qDebug() << "New location: " << LocationText;
-            if (!LocationText.isEmpty()) {
-                QUrl aURL(LocationText);
-                return go(aURL);
-            }
-            break;
-        }
-    }
-
-    ResultText = Request.lastResponse().reasonPhrase();
-    ErrorText = Request.lastResponse().value("Error");
-    return !Error;
-}
-
-bool Downloader::request(const QString& Method, const QUrl& url, const QString& Data, bool FireForget)
-{
-    if (Error) return false;
-
-    Request.setProxy(M_PREFS->getProxy(url));
-    Request.setHost(url.host(),url.port(80));
     qDebug() << "Downloader::request: " << url;
 
-    QByteArray ba(Data.toUtf8());
-    QBuffer Buf(&ba);
+    netManager.setProxy(M_PREFS->getProxy(url));
+    QNetworkRequest req(url);
 
-    QString sReq = url.toString(QUrl::RemoveScheme | QUrl::RemoveAuthority);
-    QHttpRequestHeader Header(Method,sReq);
-    Header.setValue("Host",url.host()+':'+QString::number(url.port(80)));
-    Header.setValue("User-Agent", USER_AGENT);
-    Header.setValue("Content-Type", "text/xml");
+    req.setRawHeader(QByteArray("Content-Type"), QByteArray("text/xml"));
+    req.setRawHeader(QByteArray("User-Agent"), USER_AGENT.toLatin1());
 
-    QString auth = QString("%1:%2").arg(User).arg(Password);
-    QByteArray ba_auth = auth.toUtf8().toBase64();
-    Header.setValue("Authorization", QString("Basic %1").arg(QString(ba_auth)));
+    QByteArray dataArray(theData.toUtf8());
+    QBuffer dataBuffer(&dataArray);
+    currentReply = netManager.sendCustomRequest(req, theMethod.toLatin1(), &dataBuffer);
 
-    Content.clear();
-    Id = Request.request(Header,ba);
+    if (AnimationTimer) {
+        AnimationTimer->start(200);
+        connect(currentReply,SIGNAL(downloadProgress(qint64, qint64)), this,SLOT(progress(qint64, qint64)));
+    }
 
     if (FireForget)
         return true;
 
     if (Loop.exec() == QDialog::Rejected)
-    {
-        Request.abort();
         return false;
-    }
-    Content = Request.readAll();
-    if (Request.lastResponse().hasKey("Content-encoding"))
-    {
-        QString t(Request.lastResponse().value("Content-encoding"));
-        if (t == "gzip")
-        {
-            QByteArray Uncompressed(gzipDecode(Content));
-            Content = Uncompressed;
+
+    if (AnimationTimer)
+        SAFE_DELETE(AnimationTimer);
+
+    /* Test for redirections */
+    QVariant redir = currentReply->attribute(QNetworkRequest::RedirectionTargetAttribute);
+    if (redir.isValid()) {
+        LocationText = redir.toString();
+        if (!LocationText.isEmpty()) {
+            QUrl newUrl(LocationText);
+            return request(theMethod, newUrl, theData, FireForget);
         }
     }
-#ifdef DEBUG_NONGET_CALL
-    showDebug(Method,URL,Data,Content);
-#endif
-    Result = Request.lastResponse().statusCode();
-    ResultText = Request.lastResponse().reasonPhrase();
-    ErrorText = Request.lastResponse().value("Error");
+
+    /* Handler error? */
+    if (currentReply->error())
+        QMessageBox::information(0,tr("error"), currentReply->errorString());
+
+
+    /* Read the data */
+    Content = currentReply->readAll();
+    Result = currentReply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+    ResultText = currentReply->errorString();
+    ErrorText = currentReply->rawHeader("Error");
     return !Error;
 }
 
+void Downloader::on_authenticationRequired( QNetworkReply *reply, QAuthenticator *auth) {
+    static QNetworkReply *lastReply = NULL;
+
+    /* Only provide authentication the first time we see this reply, to avoid
+     * infinite loop providing the same credentials. */
+    if (lastReply != reply) {
+        lastReply = reply;
+        qDebug() << "Downloader authentication required and provided.";
+        auth->setUser(User);
+        auth->setPassword(Password);
+    }
+}
+
 QByteArray& Downloader::content()
 {
     return Content;
 }
 
-void Downloader::on_responseHeaderReceived(const QHttpResponseHeader & hdr)
+void Downloader::on_requestFinished( QNetworkReply *reply)
 {
-    //switch (hdr.statusCode()) {
-    //	case 200:
-    //		break;
-    //	case 406:
-    //		QMessageBox::critical(NULL,QApplication::translate("MerkaartorPreferences","Preferences upload failed"), QApplication::translate("MerkaartorPreferences","Duplicate key"));
-    //		break;
-    //	case 413:
-    //		QMessageBox::critical(NULL,QApplication::translate("MerkaartorPreferences","Preferences upload failed"), QApplication::translate("MerkaartorPreferences","More than 150 preferences"));
-    //		break;
-    //	default:
-    //		qDebug() << hdr.statusCode();
-    //		qDebug() << hdr.reasonPhrase();
-    //		break;
-    //}
-
-    qDebug() << "Downloader::on_responseHeaderReceived: " << hdr.statusCode() << hdr.reasonPhrase();
-}
-
-void Downloader::on_requestFinished(int anId, bool anError)
-{
-    if (anError)
+    if (reply->error())
         Error = true;
-    if ( (anId == Id) && Loop.isRunning() )
+    if ( (reply == currentReply) && Loop.isRunning() )
         Loop.exit(QDialog::Accepted);
 }
 
-void Downloader::progress(int done, int total)
+void Downloader::progress(qint64 done, qint64 total)
 {
     if (AnimatorLabel && AnimatorBar)
     {
@@ -385,14 +220,18 @@ QString Downloader::getURLToFetch(const QString &What)
 QString Downloader::getURLToFetchFull(IFeature::FId id)
 {
     QString type;
-    if (id.type & IFeature::Point)
+	QString URL;
+    if (id.type & IFeature::Point) {
         type = "node";
-    else if (id.type & IFeature::LineString)
-        type = "way";
-    else if (id.type & IFeature::OsmRelation)
-        type = "relation";
+		URL = QString("/%1/%2");
+	} else {
+		if (id.type & IFeature::LineString)
+			type = "way";
+		if (id.type & IFeature::OsmRelation)
+			type = "relation";
+		URL = QString("/%1/%2/full");
+	}
 
-    QString URL = QString("/%1/%2/full");
     return URL.arg(type).arg(id.numId);
 }
 
@@ -470,9 +309,6 @@ bool downloadOSM(QWidget* aParent, const QUrl& theUrl, const QString& aUser, con
 #endif
         return false;
     }
-#ifdef DEBUG_MAPCALL_ONLY
-    showDebug("GET", URL,QByteArray(), Rcv.content());
-#endif
     int x = Rcv.resultCode();
     switch (x)
     {
@@ -688,10 +524,21 @@ bool downloadMapdust(MainWindow* Main, const CoordBox& aBox, Document* theDocume
 
     theDownloader.setAnimator(dlg,Lbl,Bar,true);
     Lbl->setText(QApplication::translate("Downloader","Downloading points"));
-    url.addQueryItem("t", COORD2STRING(aBox.topRight().y()));
-    url.addQueryItem("l", COORD2STRING(aBox.bottomLeft().x()));
-    url.addQueryItem("b", COORD2STRING(aBox.bottomLeft().y()));
-    url.addQueryItem("r", COORD2STRING(aBox.topRight().x()));
+
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+    QUrlQuery theQuery(url);
+#define theQuery theQuery
+#else
+#define theQuery url
+#endif
+    theQuery.addQueryItem("t", COORD2STRING(aBox.topRight().y()));
+    theQuery.addQueryItem("l", COORD2STRING(aBox.bottomLeft().x()));
+    theQuery.addQueryItem("b", COORD2STRING(aBox.bottomLeft().y()));
+    theQuery.addQueryItem("r", COORD2STRING(aBox.topRight().x()));
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+    url.setQuery(theQuery);
+#endif
+#undef theQuery
 
     if (!theDownloader.go(url))
         return false;
@@ -750,11 +597,22 @@ bool downloadOpenstreetbugs(MainWindow* Main, const CoordBox& aBox, Document* th
 
     theDownloader.setAnimator(dlg,Lbl,Bar,true);
     Lbl->setText(QApplication::translate("Downloader","Downloading points"));
-    osbUrl.addQueryItem("t", COORD2STRING(aBox.topRight().y()));
-    osbUrl.addQueryItem("l", COORD2STRING(aBox.bottomLeft().x()));
-    osbUrl.addQueryItem("b", COORD2STRING(aBox.bottomLeft().y()));
-    osbUrl.addQueryItem("r", COORD2STRING(aBox.topRight().x()));
-    osbUrl.addQueryItem("open", "yes");
+
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+    QUrlQuery theQuery(osbUrl);
+#define theQuery theQuery
+#else
+#define theQuery osbUrl
+#endif
+    theQuery.addQueryItem("t", COORD2STRING(aBox.topRight().y()));
+    theQuery.addQueryItem("l", COORD2STRING(aBox.bottomLeft().x()));
+    theQuery.addQueryItem("b", COORD2STRING(aBox.bottomLeft().y()));
+    theQuery.addQueryItem("r", COORD2STRING(aBox.topRight().x()));
+    theQuery.addQueryItem("open", "yes");
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+    osbUrl.setQuery(theQuery);
+#endif
+#undef theQuery
 
     if (!theDownloader.go(osbUrl))
         return false;
@@ -786,6 +644,7 @@ bool downloadMoreOSM(MainWindow* Main, const CoordBox& aBox , Document* theDocum
     osmWebsite = M_PREFS->getOsmApiUrl();
     osmUser = M_PREFS->getOsmUser();
     osmPwd = M_PREFS->getOsmPassword();
+    qDebug() << "Requesting more from " << osmWebsite;
 
     Main->view()->setUpdatesEnabled(false);
 
@@ -904,7 +763,7 @@ bool downloadOSM(MainWindow* Main, const CoordBox& aBox , Document* theDocument)
             if (directAPI) {
                 if (ui.FromXapi->isChecked())
                     theLayer->setUploadable(false);
-                OK = downloadOSM(Main,QUrl(QUrl::fromEncoded(directUrl.toAscii())),osmUser,osmPwd,theDocument,theLayer);
+                OK = downloadOSM(Main,QUrl(QUrl::fromEncoded(directUrl.toLatin1())),osmUser,osmPwd,theDocument,theLayer);
             }
             else
                 OK = downloadOSM(Main,osmWebsite,osmUser,osmPwd,Clip,theDocument,theLayer);
