@@ -36,6 +36,7 @@
 #include <QList>
 #include <QMenu>
 #include <QSet>
+#include <QReadWriteLock>
 
 /* MAPDOCUMENT */
 
@@ -51,6 +52,7 @@ public:
         , lastDownloadLayer(0)
         , tagFilter(0), FilterRevision(0)
         , layerNum(0)
+        , theFeaturePaintersLock( QReadWriteLock::Recursive )
     {
     };
     ~MapDocumentPrivate()
@@ -79,6 +81,7 @@ public:
     mutable QString Id;
 
     QList<FeaturePainter> theFeaturePainters;
+    QReadWriteLock theFeaturePaintersLock;
 };
 
 Document::Document()
@@ -124,11 +127,17 @@ const QString& Document::id() const
 
 void Document::setPainters(QList<Painter> aPainters)
 {
+    lockPaintersForWrite();
     p->theFeaturePainters.clear();
     for (int i=0; i<aPainters.size(); ++i) {
         FeaturePainter fp(aPainters[i]);
         p->theFeaturePainters.append(fp);
     }
+    for (FeatureIterator it(this); !it.isEnd(); ++it)
+    {
+        it.get()->invalidatePainter();
+    }
+    unlockPainters();
 }
 
 int Document::getPaintersSize()
@@ -136,6 +145,18 @@ int Document::getPaintersSize()
     return p->theFeaturePainters.size();
 }
 
+void Document::unlockPainters() {
+    p->theFeaturePaintersLock.unlock();
+}
+
+void Document::lockPainters() {
+    p->theFeaturePaintersLock.lockForRead();
+}
+
+void Document::lockPaintersForWrite() {
+    qDebug() << "LockW";
+}
+
 const Painter* Document::getPainter(int i)
 {
     return &p->theFeaturePainters[i];
@@ -375,7 +396,7 @@ FilterLayer* Document::addFilterLayer(FilterLayer *aLayer)
 {
     FilterLayer* theLayer = aLayer;
     if (!theLayer)
-        theLayer = new FilterLayer(QUuid::createUuid(), tr("Filter layer #%1").arg(++p->layerNum), "false");
+        theLayer = new FilterLayer(QUuid::createUuid().toString(), tr("Filter layer #%1").arg(++p->layerNum), "false");
     add(theLayer);
 
     FeatureIterator it(this);
@@ -1019,33 +1040,26 @@ QList<Feature*> Document::mergeDocument(Document* otherDoc, Layer* layer, Comman
 Document* Document::getDocumentFromClipboard()
 {
     QClipboard *clipboard = QApplication::clipboard();
-    QDomDocument* theXmlDoc = new QDomDocument();
+    QDomDocument theXmlDoc;
 
     if (clipboard->mimeData()->hasFormat("application/x-openstreetmap+xml")) {
-        if (!theXmlDoc->setContent(clipboard->mimeData()->data("application/x-openstreetmap+xml"))) {
-            delete theXmlDoc;
+        if (!theXmlDoc.setContent(clipboard->mimeData()->data("application/x-openstreetmap+xml"))) {
             return NULL;
         }
     } else
     if (clipboard->mimeData()->hasFormat("application/vnd.google-earth.kml+xml")) {
-        if (!theXmlDoc->setContent(clipboard->mimeData()->data("application/vnd.google-earth.kml+xml"))) {
-            delete theXmlDoc;
+        if (!theXmlDoc.setContent(clipboard->mimeData()->data("application/vnd.google-earth.kml+xml"))) {
             return NULL;
         }
     } else
     if (clipboard->mimeData()->hasText()) {
-        if (!theXmlDoc->setContent(clipboard->text())) {
-            delete theXmlDoc;
+        if (!theXmlDoc.setContent(clipboard->text())) {
             return NULL;
         }
     } else {
-        delete theXmlDoc;
         return NULL;
     }
-    Document* doc = Document::getDocumentFromXml(theXmlDoc);
-    delete theXmlDoc;
-
-    return doc;
+    return Document::getDocumentFromXml(&theXmlDoc);
 }
 
 /* FEATUREITERATOR */
