@@ -435,12 +435,12 @@ static void splitRoad(Document* theDocument, CommandList* theList, Way* In, cons
         }
     }
 
-    Way* FirstPart = In;
-    Result.push_back(FirstPart);
+    Way* FirstPart     = In;
+    bool FirstWayValid = false;
     for (int i=1; (i+1)<FirstPart->size(); ++i)
     {
-        if (std::find(Points.begin(),Points.end(),FirstPart->get(i)) != Points.end())
-        {
+        if (std::find(Points.begin(),Points.end(),FirstPart->get(i)) != Points.end() and FirstPart->get(i) != FirstPart->get(i+1) \
+                and (FirstWayValid or FirstPart->get(0) != FirstPart->get(i))) {
             Way* NextPart = g_backend.allocWay(theDocument->getDirtyOrOriginLayer(In->layer()));
             theList->add(new AddFeatureCommand(theDocument->getDirtyOrOriginLayer(In->layer()),NextPart,true));
             copyTags(NextPart,FirstPart);
@@ -452,12 +452,13 @@ static void splitRoad(Document* theDocument, CommandList* theList, Way* In, cons
             }
             handleWaysplitRelations(theDocument, theList, FirstPart, NextPart);
 
-            Result.push_back(NextPart);
+            FirstWayValid = true;
+            Result.push_back(FirstPart);
             FirstPart = NextPart;
             i=0;
         }
     }
-
+    Result.push_back(FirstPart);
 
 }
 
@@ -483,6 +484,32 @@ void splitRoads(Document* theDocument, CommandList* theList, PropertiesDock* the
     theDock->setSelection(Result);
 }
 
+/* Split road by the two nodes and return the way joining them, if there is one. */
+Way *cutoutRoad(Document* theDocument, CommandList* theList, PropertiesDock* /* theDock */, Node *N1, Node *N2) {
+    QList<Way*> Roads, Result;
+    QList<Node*> Points;
+
+	Way *R1 = Way::GetSingleParentRoadInner( N1 );
+	Way *R2 = Way::GetSingleParentRoadInner( N2 );
+
+	if (R1)
+		Roads.push_back(R1);
+	if (R2)
+		Roads.push_back(R2);
+
+    Points.push_back(N1);
+    Points.push_back(N2);
+
+    for (int i=0; i<Roads.size(); ++i)
+        splitRoad(theDocument, theList, Roads[i], Points, Result);
+
+	for (int i = 0; i < Result.size(); ++i) {
+		if (Result[i]->isExtrimity(N1) && Result[i]->isExtrimity(N2))
+			return Result[i];
+	}
+	return NULL;
+}
+
 static void breakRoad(Document* theDocument, CommandList* theList, Way* R, Node* Pt)
 {
     for (int i=0; i<R->size(); ++i)
@@ -1935,12 +1962,10 @@ AxisAlignResult axisAlignRoads(Document* theDocument, CommandList* theList, Prop
         // tweak midpoints
         i = 0;
         foreach (Way *theWay, theWays) {
-            Node *n1;
             Node *n2 = theWay->getNode(0);
             QPointF p1;
             QPointF p2 = node_pos[n2];
             for (int j = 0; j < theWay->size()-1; ++j, ++i) {
-                n1 = n2;
                 n2 = theWay->getNode(j + 1);
                 p1 = p2;
                 p2 = node_pos[n2];
